# -*- coding: utf-8 -*-
## Description: utility functions used while loading NeuroML L1,2,3 files.
## Version 1.0 by Aditya Gilra, NCBS, Bangalore, India, 2011 for serial MOOSE
## Version 1.5 by Niraj Dudani, NCBS, Bangalore, India, 2012, modified for parallel MOOSE
## Version 1.6 by Aditya Gilra, NCBS, Bangalore, India, 2012, minor changes for parallel MOOSE

"""
Some useful constants like namespaces are defined.
And they can be set in ElementTree root element via set_neuroml_namespaces_attribs(neuromlroot).
Use tweak_model(root_element, params) to exclude certain populations and projections
while still including certain others.
indent(...) is an in-place prettyprint formatter copied from http://effbot.org/zone/element-lib.htm
"""

from __future__ import print_function
from xml.etree import cElementTree as ET
from xml.etree import ElementTree as slowET
import math
import os

neuroml_debug = False

neuroml_ns='http://morphml.org/neuroml/schema'
nml_ns='http://morphml.org/networkml/schema'
mml_ns='http://morphml.org/morphml/schema'
bio_ns='http://morphml.org/biophysics/schema'
cml_ns='http://morphml.org/channelml/schema'
meta_ns='http://morphml.org/metadata/schema'
xsi_ns='http://www.w3.org/2001/XMLSchema-instance'

### ElementTree parse works an order of magnitude or more faster than minidom
### BUT it doesn't keep the original namespaces,
## from http://effbot.org/zone/element-namespaces.htm , I got _namespace_map
## neuroml_ns, bio_ns, mml_ns, etc are defined above
slowET._namespace_map[neuroml_ns] = 'neuroml'
slowET._namespace_map[nml_ns] = 'nml'
slowET._namespace_map[mml_ns] = 'mml'
slowET._namespace_map[bio_ns] = 'bio'
slowET._namespace_map[cml_ns] = 'cml'
slowET._namespace_map[meta_ns] = 'meta'
slowET._namespace_map[xsi_ns] = 'xsi'

### cElementTree is much faster than ElementTree and is API compatible with the latter,
### but instead of _namespace_map above, use register_namespace below ...
### but this works only with python2.7 onwards, so stick to above,
### with import elementtree.ElementTree alongwith importing cElementTree as at
### http://dev.blogs.nuxeo.com/2006/02/elementtree-serialization-namespace-prefixes.html
#ET.register_namespace('neuroml',neuroml_ns)
#ET.register_namespace('nml',nml_ns)
#ET.register_namespace('mml',mml_ns)
#ET.register_namespace('bio',bio_ns)
#ET.register_namespace('cml',cml_ns)
#ET.register_namespace('meta',meta_ns)
#ET.register_namespace('xsi',xsi_ns)

CELSIUS_default = 32.0 # deg C # default temperature if meta:property tag for temperature is not present
ZeroCKelvin = 273.15 # zero dec C in Kelvin
VMIN = -0.1 # Volts
VMAX = 0.1 # Volts
NDIVS = 200 # number
dv = ( VMAX - VMIN ) / NDIVS # Volts

def set_neuroml_namespaces_attribs(neuromlroot):
    set_attrib_if_not_found(neuromlroot,"xmlns",neuroml_ns)
    set_attrib_if_not_found(neuromlroot,"xmlns:nml",nml_ns)
    set_attrib_if_not_found(neuromlroot,"xmlns:mml",mml_ns)
    set_attrib_if_not_found(neuromlroot,"xmlns:bio",bio_ns)
    set_attrib_if_not_found(neuromlroot,"xmlns:cml",cml_ns)
    set_attrib_if_not_found(neuromlroot,"xmlns:meta",meta_ns)
    ## later doc.write() assigns the xsi namespace a second time
    ## causing double definition and problem with xsltproc,
    ## hence commenting it out here.
    #set_attrib_if_not_found(neuromlroot,"xmlns:xsi",xsi_ns)

def set_attrib_if_not_found(elem, name, value):
    if elem.get(name) is None:
        elem.set(name,value)

def tweak_model(root_element, params):
    if 'excludePopulations' in params: # if params has key 'excludePopulations'
        ## both excludePopulations and excludeProjections must be present together
        pruneExcludes(root_element,params['excludePopulations'],params['excludeProjections'])
    if 'onlyInclude' in params: # if params has key 'onlyInclude'
        keepOnlyInclude(root_element,params['onlyInclude'])

def pruneExcludes(network, excludepops, excludeprojs):
    """
    remove the populations in the excludepops list
    remove the projections in the excludeprojs list
    """
    populations = network.find(".//{"+nml_ns+"}populations")
    pop_remove_list = []
    for population in populations.findall(".//{"+nml_ns+"}population"):
        populationname = population.attrib["name"]
        ## if any of the name-s in exclude_list are a SUBSTRING
        ## of the name of the neuroml population, mark it for removal
        for name in excludepops:
            if name in populationname: # substring
                pop_remove_list.append(population)
    ## remove only unique names,
    ## else you could end up trying to remove same population twice
    for population in set(pop_remove_list):
        populations.remove(population)

    projections = network.find(".//{"+nml_ns+"}projections")
    proj_remove_list = []
    for projection in projections.findall(".//{"+nml_ns+"}projection"):
        projectionname = projection.attrib["name"]
        ## if any of the name-s in exclude_list are a SUBSTRING
        ## of the name of the neuroml projection, mark it for removal
        for name in excludeprojs:
            if name in projectionname: # substring
                proj_remove_list.append(projection)
    ## remove only unique names,
    ## else you could end up trying to remove same projection twice
    for projection in set(proj_remove_list):
        projections.remove(projection)

def keepOnlyInclude(network, onlyInclude):
    """
    Keep only the cells that are in onlyInclude['includePopulation']
    and also keep cells that are connected to cells in onlyInclude['includePopulation']
    and keep connections to any of the cells in onlyInclude['includePopulation'].
    Prune the extraneous connections
    but keep those connections in onlyInclude['includeProjections']
    on cells connected to those in onlyInclude['includePopulation']
    """
    ### Remove the connections that do not connect to cells in onlyInclude.
    ### Simultaneously build up a list of cells 'includeCellsDict' that connect to cells in onlyInclude.
    ### Of course this includeCellDict must have the originally included cells!
    ### At the end of this pruning, even if some population-s / projection-s have no elements,
    ### it doesn't matter, as this findall() returns an empty list and not None - so no error.
    ### Further I am not changing the 'size' attrib in <instances> and <connections>,
    ### as it's not used by this reader and I'm not saving the network after pruning.
    ### Do not prune 'includeProjections' immediately;
    ### prune them later avoiding second order cells in includeCellsDict.
    includepopname = onlyInclude['includePopulation'][0]
    includecellids = onlyInclude['includePopulation'][1]
    ## first of all, include those primary cells that the user instructs.
    includeCellsDict = {includepopname:includecellids}
    ## projections 'includeProjs' will be pruned later, keeping connections to second order cells.
    includeProjs = []
    print("removing obviously extra connections in ... ")
    for projection in network.findall(".//{"+nml_ns+"}projection"):
        projname = projection.attrib['name']
        includeProj = False
        ## check if any of the given includeprojname is a substring of this projname
        for includeprojname in onlyInclude['includeProjections']:
            if includeprojname in projname:
                includeProj = True
        ## if it is a substring, add this projection
        ## to the list of projections to be pruned later
        if includeProj:
            includeProjs.append(projection)
        source = projection.attrib["source"]
        target = projection.attrib["target"]
        print(projname, source, target)
        connections = projection.find(".//{"+nml_ns+"}connections")
        if connections is not None:
            for connection in connections.findall(".//{"+nml_ns+"}connection"):
                pre_cell_id = connection.attrib['pre_cell_id']
                ## is the user-included cell a source cell of the connection?
                includecellinsource = (pre_cell_id in includecellids and includepopname==source)
                post_cell_id = connection.attrib['post_cell_id']
                ## is the user-included cell a target cell of the connection?
                includecellintarget = (post_cell_id in includecellids and includepopname==target)
                ## the second-order cell connected to the user-included cell must also be kept
                if includecellinsource:
                    ## since source is included, include the target also
                    ## there can be self connections between the same population i.e. same source and target
                    try:
                        includeCellsDict[target].append(post_cell_id)
                    except KeyError: # create this population entry in the dictionary if not present
                        includeCellsDict[target] = [post_cell_id]
                elif includecellintarget:
                    ## since target is included, include the source also, except if source is a file
                    if 'file' not in source:
                        try:
                            includeCellsDict[source].append(pre_cell_id)
                        except KeyError: # create this population entry in the dictionary if not present
                            includeCellsDict[source] = [pre_cell_id]
                else:
                    ## this connection is extraneous
                    ## but remove this connection only if
                    ## it is not part of the projections to be pruned later
                    if not includeProj:
                        connections.remove(connection)

    ## convert includeCellsDict elements to set-s rather than lists
    ## to have only unique cell_ids and save time below.
    for key in includeCellsDict:
        includeCellsDict[key] = set(includeCellsDict[key])

    print("removing extra cells ... ")
    ### remove the cells that are not in includeCellsDict
    populations = network.find(".//{"+nml_ns+"}populations")
    for population in network.findall(".//{"+nml_ns+"}population"):
        popname = population.attrib["name"]
        if popname in includeCellsDict:
            includecellids = includeCellsDict[popname]
            instances = population.find(".//{"+nml_ns+"}instances")
            for instance in instances.findall(".//{"+nml_ns+"}instance"):
                ## not a connected cell, so remove
                if instance.attrib['id'] not in includecellids:
                    instances.remove(instance)
        else: ## this whole population is not required!
            populations.remove(population)

    ### Prune the 'includeProjections' that we skipped pruning before,
    ### while keeping connections to second order cells!
    for projection in includeProjs:
        print("removing projection",projection.attrib['name'],\
            "keeping second-order connections.")
        source = projection.attrib["source"]
        target = projection.attrib["target"]
        ## boolean: True if includeCellsDict has key source
        source_in_includeCellsDict = source in includeCellsDict
        ## boolean: True if the word 'file' occurs in str source
        file_in_source = 'file' in source
        ## boolean: True if includeCellsDict has key target
        target_in_includeCellsDict = target in includeCellsDict
        connections = projection.find(".//{"+nml_ns+"}connections")
        for connection in connections.findall(".//{"+nml_ns+"}connection"):
            ## is the included cell a source cell of the connection?
            ## keep 'file' as source also.
            if file_in_source:
                includecellinsource = True
            elif source_in_includeCellsDict and \
                    connection.attrib['pre_cell_id'] in includeCellsDict[source]:
                includecellinsource = True
            else: includecellinsource = False
            ## is the included cell a target cell of the connection?
            if target_in_includeCellsDict and \
                    connection.attrib['post_cell_id'] in includeCellsDict[target]:
                includecellintarget = True
            else: includecellintarget= False
            ## this connection is extraneous
            ## if either sourcecell or targetcell is not included.
            if not includecellinsource or not includecellintarget:
                ## remove is a very slow operation!
                connections.remove(connection)

def indent(elem, level=0):
    """ in-place prettyprint formatter copied from http://effbot.org/zone/element-lib.htm
    first call indent(root, level=0), and then doc.write(filename) ."""
    i = "\n" + level*"  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
        for elem in elem:
            indent(elem, level+1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i

## make a list of safe functions possible to be used safely in eval()
safe_list = ('acos', 'asin', 'atan', 'atan2', 'ceil',
    'cos', 'cosh', 'degrees', 'e', 'exp', 'fabs', 'floor',
    'fmod', 'frexp', 'hypot', 'ldexp', 'log', 'log10', 'modf',
    'pi', 'pow', 'radians', 'sin', 'sinh', 'sqrt', 'tan', 'tanh')
## use the list to filter the local namespace
safe_dict = {k:getattr(math, k) for k in safe_list}
## add any needed builtins back in.
safe_dict['abs'] = abs

def find_first_file(name, path):
    """ Finds and returns the first occurence of the filename in the directory tree under a given path.
    If nothing is returned, return value defaults to None.  """
    for root, dirs, files in os.walk(path):
        if name in files:
            return os.path.join(root, name)



# -*- coding: utf-8 -*-
"""
Test SBML capabilities of PyMOOSE
"""

__author__ = "Dilawar Singh, HarshaRani"
__copyright__ = "Copyright 2015, Dilawar Singh and NCBS Bangalore"
__credits__ = ["NCBS Bangalore"]
__license__ = "GNU GPL"
__version__ = "1.0.0"
__maintainer__ = "Dilawar Singh"
__email__ = "dilawars@ncbs.res.in"
__status__ = "Development"

import os
import moose

print("Using moose form %s" % moose.__file__)
print("\t Moose version %s" % moose.version())

# the model lives in the same directory as the test script
sdir_ = os.path.dirname(os.path.realpath(__file__))


def test_sbml():
    modelname = os.path.join(sdir_, "..", "data", "00001-sbml-l3v1.xml")
    model = moose.readSBML(modelname, "/sbml")
    if not model:
        print("Most likely python-libsbml is not installed.")
        return 0
    moose.reinit()
    moose.start(200)
    # TODO: Add more tests here.
    p = moose.wildcardFind("/sbml/##")
    assert len(p) == 8, p
    names = ["compartment", "mesh", "S1", "info", "S2", "info", "reaction1"]
    for x in p:
        assert x.name in names


def main():
    test_sbml()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Issue 34 on github.
import moose
import moose.utils as mu

compt = moose.CubeMesh('/compt')
compt.volume = 1e-20

molecules = [ "ca", "S", "S1", "Sp" ]
pools = {}
tables = {}
for m in molecules:
    pools[m] = moose.Pool('/compt/%s' % m)
    t = moose.Table2("/table%s" % m)
    tables[m] = t
    moose.connect(t, 'requestOut', pools[m], 'getConc')

pools['ca'].nInit = 20

r_p0_p1 = moose.Reac('/compt/reacA')
funA = moose.Function('/compt/funA')
funA.x.num = 1
funA.expr = "{0}*(x0/{1})^6/(1+(x0/{1})^3)^2".format("1.5", "0.7e-3")
# moose.connect(funA.x[0], 'input',  pools['ca'], 'getN')
moose.connect(pools['ca'], 'nOut', funA.x[0], 'input')
moose.connect(funA, 'valueOut', pools['S1'], 'setN')
moose.connect(r_p0_p1, 'sub', pools['S'], 'reac')
moose.connect(r_p0_p1, 'prd', pools['S1'], 'reac')

r_p1_up = moose.Reac('/compt/reacB')
moose.connect(r_p1_up, 'sub', pools['S1'], 'reac')
moose.connect(r_p1_up, 'prd', pools['Sp'], 'reac')

# Change GSolve to Ksolve and bugs go away.
k = moose.Gsolve('/compt/ksolve')
s = moose.Stoich('/compt/stoich')
s.compartment = compt
s.ksolve = k
s.path = '/compt/##'

moose.reinit()
moose.start(10)



# -*- coding: utf-8 -*-

# Top level file to run and generate report.

__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2019-, Dilawar Singh"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"

import pytest
try:
    import reload
except ImportError:
    from importlib import reload

@pytest.fixture(scope="module", autouse=True)
def reload_moose():
    import moose
    print('reloading moose')
    for p in ['/model', '/library']:
        if moose.exists(p):
            moose.delete(p)
    # If someone has changed the clock-ticks then we need to reset them. 
    # And relaod just to make sure and other values are initialized. 
    reload(moose)



# -*- coding: utf-8 -*-
"""\
Create general Channel Proto, pass in name, x and y power, and params

Also, create the library of channels
Might need a few other chan_proto types, such as
     inf-tau channels
     Ca dep channels
chan_proto quires alpha and beta params for both activation and inactivation
If no inactivation, just send in empty Yparam array.
"""

from __future__ import print_function, division
import moose
import numpy as np
from collections import namedtuple

ChannelSettings = namedtuple('''ChannelSettings''', '''
Xpow
Ypow
Zpow
Erev
name''')
ZChannelParams = namedtuple('''ZChannelParams''',
                           '''Kd
                           power
                           tau''')

ChannelParams = namedtuple('''ChannelParams''', '''channel
X
Y
Z
''')

GateParams = namedtuple('''GateParams''', '''
A_rate
A_B
A_C
Avhalf
A_vslope
B_rate
B_B
B_C
Bvhalf
B_vslope''')


def interpolate_values_in_table(tabA,V_0,l=40):
    import param_chan
    '''This function interpolates values in the table
    around tabA[V_0]. '''
    V = np.linspace(param_chan.VMIN, param_chan.VMAX, len(tabA))
    idx =  abs(V-V_0).argmin()
    A_min = tabA[idx-l]
    V_min = V[idx-l]
    A_max = tabA[idx+l]
    V_max = V[idx+l]
    a = (A_max-A_min)/(V_max-V_min)
    b = A_max - a*V_max
    tabA[idx-l:idx+l] = V[idx-l:idx+l]*a+b
    return tabA

def fix_singularities(Params,Gate):
    import param_chan

    if Params.A_C < 0:

        V_0 = Params.A_vslope*np.log(-Params.A_C)-Params.Avhalf

        if V_0 > param_chan.VMIN and V_0 < param_chan.VMAX:
            #change values in tableA and tableB, because tableB contains sum of alpha and beta
            tabA = interpolate_values_in_table(Gate.tableA,V_0)
            tabB = interpolate_values_in_table(Gate.tableB,V_0)
            Gate.tableA = tabA
            Gate.tableB = tabB

    if Params.B_C < 0:

        V_0 = Params.B_vslope*np.log(-Params.B_C)-Params.Bvhalf

        if V_0 > param_chan.VMIN and V_0 < param_chan.VMAX:
            #change values in tableB
            tabB = interpolate_values_in_table(Gate.tableB,V_0)
            Gate.tableB = tabB

    return Gate

#may need a CaV channel if X gate uses alpha,beta and Ygate uses inf tau
#Or, have Y form an option - if in tau, do something like NaF
def chan_proto(chanpath, params):
    import param_chan

    chan = moose.HHChannel(chanpath)
    chan.Xpower = params.channel.Xpow
    if params.channel.Xpow > 0:
        xGate = moose.HHGate(chan.path + '/gateX')
        xGate.setupAlpha(params.X + (param_chan.VDIVS, param_chan.VMIN, param_chan.VMAX))
        xGate = fix_singularities(params.X, xGate)

    chan.Ypower = params.channel.Ypow
    if params.channel.Ypow > 0:
        yGate = moose.HHGate(chan.path + '/gateY')
        yGate.setupAlpha(params.Y + (param_chan.VDIVS, param_chan.VMIN, param_chan.VMAX))
        yGate = fix_singularities(params.Y, yGate)
    if params.channel.Zpow > 0:
        chan.Zpower = params.channel.Zpow
        zgate = moose.HHGate(chan.path + '/gateZ')
        ca_array = np.linspace(param_chan.CAMIN, param_chan.CAMAX, param_chan.CADIVS)
        zgate.min = param_chan.CAMIN
        zgate.max = param_chan.CAMAX
        caterm = (ca_array/params.Z.Kd) ** params.Z.power
        inf_z = caterm / (1 + caterm)
        tau_z = params.Z.tau * np.ones(len(ca_array))
        zgate.tableA = inf_z / tau_z
        zgate.tableB = 1 / tau_z
        chan.useConcentration = True
    chan.Ek = params.channel.Erev
    return chan





# Moogul.py: MOOSE Graphics Using Lines
# This is a fallback graphics interface for displaying neurons using
# regular matplotlib routines.
# Put in because the GL versions like moogli need all sorts of difficult 
# libraries and dependencies.
# Copyright (C) Upinder S. Bhalla NCBS 2018
# This program is licensed under the GNU Public License version 3.
#

import moose
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d.art3d import Line3DCollection

class MoogulError( Exception ):
    def __init__( self, value ):
        self.value = value
    def __str__( self ):
        return repr( self.value )

class MooView:
    ''' The MooView class is a window in which to display one or more 
    moose cells, using the MooCell class.'''
    def __init__( self, swx = 10, swy = 12, hideAxis = True
    ):
        plt.ion()
        self.fig_ = plt.figure( figsize = (swx, swy) )
        self.ax = self.fig_.add_subplot(111, projection='3d' )
        self.drawables_ = []
        self.fig_.canvas.mpl_connect("key_press_event", self.moveView )
        plt.rcParams['keymap.xscale'] = ''
        plt.rcParams['keymap.yscale'] = ''
        plt.rcParams['keymap.zoom'] = ''
        plt.rcParams['keymap.back'] = ''
        plt.rcParams['keymap.home'] = ''
        plt.rcParams['keymap.forward'] = ''
        plt.rcParams['keymap.all_axes'] = ''
        self.hideAxis = hideAxis
        if self.hideAxis:
            self.ax.set_axis_off()
        #self.ax.margins( tight = True )
        self.ax.margins()
        self.sensitivity = 7.0 # degrees rotation
        self.zoom = 1.05

    def addDrawable( self, n ):
        self.drawables_.append( n )

    def firstDraw( self, rotation = 0.0, elev = 0.0, azim = 0.0 ):
        self.coordMax = 0.0 
        self.coordMin = 0.0
        if rotation == 0.0:
            self.doRotation = False
            self.rotation = 7 # default rotation per frame, in degrees.
        else:
            self.doRotation = True
            self.rotation = rotation * 180/np.pi # arg units: radians/frame
        
        self.azim = azim * 180/np.pi
        self.elev = elev * 180/np.pi
        for i in self.drawables_:
            cmax, cmin = i.drawForTheFirstTime( self.ax )
            self.coordMax = max( cmax, self.coordMax )
            self.coordMin = min( cmin, self.coordMin )
        if self.coordMin == self.coordMax:
            self.coordMax = 1+self.coordMin

            
        self.ax.set_xlim3d( self.coordMin, self.coordMax )
        self.ax.set_ylim3d( self.coordMin, self.coordMax )
        self.ax.set_zlim3d( self.coordMin, self.coordMax )
        self.ax.view_init( elev = self.elev, azim = self.azim )
        #self.ax.view_init( elev = -80.0, azim = 90.0 )
        #self.colorbar = plt.colorbar( self.drawables_[0].segments )
        self.colorbar = self.fig_.colorbar( self.drawables_[0].segments )
        self.colorbar.set_label( self.drawables_[0].fieldInfo[3])
        self.timeStr = self.ax.text2D( 0.05, 0.05, 
                "Time= 0.0", transform=self.ax.transAxes)
        self.fig_.canvas.draw()
        plt.show()

    def updateValues( self ):
        time = moose.element( '/clock' ).currentTime
        self.timeStr.set_text( "Time= " + str(time) )
        for i in self.drawables_:
            i.updateValues()
        if self.doRotation and abs( self.rotation ) < 120:
            self.ax.azim += self.rotation
        #self.fig_.canvas.draw()
        plt.pause(0.001)

    def moveView(self, event):
        x0 = self.ax.get_xbound()[0]
        x1 = self.ax.get_xbound()[1]
        xk = (x0 - x1) / self.sensitivity
        y0 = self.ax.get_ybound()[0]
        y1 = self.ax.get_ybound()[1]
        yk = (y0 - y1) / self.sensitivity
        z0 = self.ax.get_zbound()[0]
        z1 = self.ax.get_zbound()[1]
        zk = (z0 - z1) / self.sensitivity

        if event.key == "up" or event.key == "k":
            self.ax.set_ybound( y0 + yk, y1 + yk )
        if event.key == "down" or event.key == "j":
            self.ax.set_ybound( y0 - yk, y1 - yk )
        if event.key == "left" or event.key == "h":
            self.ax.set_xbound( x0 + xk, x1 + xk )
        if event.key == "right" or event.key == "l":
            self.ax.set_xbound( x0 - xk, x1 - xk )
        if event.key == "ctrl+up":
            self.ax.set_zbound( z0 + zk, z1 + zk )
        if event.key == "ctrl+down":
            self.ax.set_zbound( z0 - zk, z1 - zk )
        if event.key == "." or event.key == ">": # zoom in, bigger
            self.ax.set_xbound( x0/self.zoom, x1/self.zoom )
            self.ax.set_ybound( y0/self.zoom, y1/self.zoom )
            self.ax.set_zbound( z0/self.zoom, z1/self.zoom )
        if event.key == "," or event.key == "<": # zoom out, smaller
            self.ax.set_xbound( x0*self.zoom, x1*self.zoom )
            self.ax.set_ybound( y0*self.zoom, y1*self.zoom )
            self.ax.set_zbound( z0*self.zoom, z1*self.zoom )
        if event.key == "a": # autoscale to fill view.
            self.ax.set_xlim3d( self.coordMin, self.coordMax )
            self.ax.set_ylim3d( self.coordMin, self.coordMax )
            self.ax.set_zlim3d( self.coordMin, self.coordMax )
        if event.key == "p": # pitch
            self.ax.elev += self.sensitivity
        if event.key == "P":
            self.ax.elev -= self.sensitivity
        if event.key == "y": # yaw
            self.ax.azim += self.sensitivity
        if event.key == "Y":
            self.ax.azim -= self.sensitivity
        # Don't have anything for roll
        if event.key == "g":
            self.hideAxis = not self.hideAxis
            if self.hideAxis:
                self.ax.set_axis_off()
            else:
                self.ax.set_axis_on()
        if event.key == "t": # Turn on/off twisting/autorotate
            self.doRotation = not self.doRotation
        if event.key == "?": # Print out help for these commands
            self.printMoogulHelp()

        self.fig_.canvas.draw()

    def printMoogulHelp( self ):
        print( '''
            Key bindings for Moogul:
            Up or k:    pan object up
            Down or j:  pan object down
            left or h:  pan object left. Bug: direction depends on azimuth.
            right or l:  pan object right Bug: direction depends on azimuth
            . or >:     Zoom in: make object appear bigger
            , or <:     Zoom out: make object appear smaller
            a:          Autoscale to fill view
            p:          Pitch down
            P:          Pitch up
            y:          Yaw counterclockwise
            Y:          Yaw counterclockwise
            g:          Toggle visibility of grid
            t:          Toggle turn (rotation along long axis of cell)
            ?:          Print this help page.
        ''')

#####################################################################

class MooDrawable:
    ''' Base class for drawing things'''
    def __init__( self,
        fieldInfo, field, relativeObj, maxLineWidth,
        colormap,
        lenScale, diaScale, autoscale,
        valMin, valMax
    ):
        self.field = field
        self.relativeObj = relativeObj
        self.maxLineWidth = maxLineWidth
        self.lenScale = lenScale
        self.diaScale = diaScale
        self.colormap = colormap
        self.autoscale = autoscale
        self.valMin = valMin
        self.valMax = valMax
        self.fieldInfo = fieldInfo
        self.fieldScale = fieldInfo[2]
        #FieldInfo = [baseclass, fieldGetFunc, scale, axisText, min, max]

    def updateValues( self ):
        ''' Obtains values from the associated cell'''
        self.val = np.array([moose.getField(i, self.field) for i in self.activeObjs]) * self.fieldScale
        cmap = plt.get_cmap( self.colormap )
        if self.autoscale:
            valMin = min( self.val )
            valMax = max( self.val )
        else:
            valMin = self.valMin
            valMax = self.valMax
        scaleVal = (self.val - valMin) / (valMax - valMin)
        self.rgba = [ cmap(i) for i in scaleVal ]
        self.segments.set_color( self.rgba )
        return

    def drawForTheFirstTime( self, ax ):
        self.segments = Line3DCollection( self.activeCoords, 
                linewidths = self.linewidth, cmap = plt.get_cmap(self.colormap) )
        self.cax = ax.add_collection3d( self.segments )
        self.segments.set_array( self.valMin + np.array( range( len(self.activeCoords) ) ) * (self.valMax-self.valMin)/len(self.activeCoords) )
        return self.coordMax, self.coordMin




#####################################################################

class MooNeuron( MooDrawable ):
    ''' Draws collection of line segments of defined dia and color'''
    def __init__( self,
        neuronId,
        fieldInfo,
        field = 'Vm', 
        relativeObj = '.', 
        maxLineWidth = 20, 
        colormap = 'jet', 
        lenScale = 1e6, diaScale = 1e6, autoscale = False, 
        valMin = -0.1, valMax = 0.05,
    ):
        #self.isFieldOnCompt = 
            #field in ( 'Vm', 'Im', 'Rm', 'Cm', 'Ra', 'inject', 'diameter' )
        
        MooDrawable.__init__( self, fieldInfo, field = field, 
                relativeObj = relativeObj, maxLineWidth = maxLineWidth, 
                colormap = colormap, lenScale = lenScale, 
                diaScale = diaScale, autoscale = autoscale, 
                valMin = valMin, valMax = valMax )
        self.neuronId = neuronId
        self.updateCoords()

    def updateCoords( self ):
        ''' Obtains coords from the associated cell'''
        self.compts_ = moose.wildcardFind( self.neuronId.path + "/#[ISA=CompartmentBase]" )
        # Matplotlib3d isn't able to do full rotations about an y axis,
        # which is what the NeuroMorpho models use, so
        # here we shuffle the axes around. Should be an option.
        #coords = np.array([[[i.x0,i.y0,i.z0],[i.x,i.y,i.z]] 
            #for i in self.compts_])
        coords = np.array([[[i.z0,i.x0,i.y0],[i.z,i.x,i.y]] 
            for i in self.compts_])
        dia = np.array([i.diameter for i in self.compts_])
        if self.relativeObj == '.':
            self.activeCoords = coords
            self.activeDia = dia
            self.activeObjs = self.compts_
        else:
            self.activeObjs = []
            self.activeCoords = []
            self.activeDia = []
            for i,j,k in zip( self.compts_, coords, dia ):
                if moose.exists( i.path + '/' + self.relativeObj ):
                    elm = moose.element( i.path + '/' + self.relativeObj )
                    self.activeObjs.append( elm )
                    self.activeCoords.append( j )
                    self.activeDia.append( k )

        self.activeCoords = np.array( self.activeCoords ) * self.lenScale
        self.coordMax = np.amax( self.activeCoords )
        self.coordMin = np.amin( self.activeCoords )
        self.linewidth = np.array( [ min(self.maxLineWidth, 1 + int(i * self.diaScale )) for i in self.activeDia ] )

        return

#####################################################################
class MooReacSystem( MooDrawable ):
    ''' Draws collection of line segments of defined dia and color'''
    def __init__( self,
        mooObj, fieldInfo,
        field = 'conc', 
        relativeObj = '.', 
        maxLineWidth = 100, 
        colormap = 'jet', 
        lenScale = 1e6, diaScale = 20e6, autoscale = False, 
        valMin = 0.0, valMax = 1.0
    ):
        
        MooDrawable.__init__( self, fieldInfo, field = field, 
                relativeObj = relativeObj, maxLineWidth = maxLineWidth, 
                colormap = colormap, lenScale = lenScale, 
                diaScale = diaScale, autoscale = autoscale, 
                valMin = valMin, valMax = valMax )
        self.mooObj = mooObj
        self.updateCoords()

    def updateCoords( self ):
        ''' For now a dummy cylinder '''
        dx = 1e-6
        dummyDia = 20e-6
        numObj = len( self.mooObj )
        x = np.arange( 0, (numObj+1) * dx, dx )
        y = np.zeros( numObj + 1)
        z = np.zeros( numObj + 1)

        coords = np.array([[[i*dx,0,0],[(i+1)*dx,0,0]] for i in range( numObj )] )
        dia = np.ones( numObj ) * dummyDia
        self.activeCoords = coords
        self.activeDia = dia
        self.activeObjs = self.mooObj
        self.activeCoords = np.array( self.activeCoords ) * self.lenScale
        self.coordMax = np.amax( self.activeCoords )
        self.coordMin = np.amin( self.activeCoords )
        self.linewidth = np.array( [ min(self.maxLineWidth, 1 + int(i * self.diaScale )) for i in self.activeDia ] )

        return



# -*- coding: utf-8 -*-
""" Chemical Signalling model loaded into moose can be save into Genesis-Kkit format """

__author__           = "Harsha Rani"
__copyright__        = "Copyright 2017, Harsha Rani and NCBS Bangalore"
__credits__          = ["NCBS Bangalore"]
__license__          = "GNU GPL"
__version__          = "1.0.0"
__maintainer__       = "Harsha Rani"
__email__            = "hrani@ncbs.res.in"
__status__           = "Development"
__updated__          = "Jan 08 2020"

#Jan 8: added a line to add compartment info
#       permeability from moose is in uM which to be converted to mM for genesis
#2020
# 2018
# Dec 08: using restoreXreacs from fixXreacs 
# Nov 22: searched for _xfer_ instead of xfer
# Nov 21: xfer pool are not written and cleaned up if part of Reaction/Enzyme or notes, 
#         group are checked under all the mesh
          
# Oct 16: Channels are written back to genesis
#         only zeroth element is taken for written back to genesis, this is true for CubeMesh and CylMesh
# 2017
# Aug 8 : All the moose object which doesn't have compartment are not written. Error message are appended

import sys
import random
import re
import moose
from moose.chemUtil.chemConnectUtil import *
from moose.chemUtil.graphUtils import *
from moose.fixXreacs import restoreXreacs
        
foundmatplotlib_ = False
try:
    import matplotlib
    foundmatplotlib_ = True
except Exception as e:
    pass

GENESIS_COLOR_SEQUENCE = ((248, 0, 255), (240, 0, 255), (232, 0, 255), (224, 0, 255), (216, 0, 255), (208, 0, 255),
 (200, 0, 255), (192, 0, 255), (184, 0, 255), (176, 0, 255), (168, 0, 255), (160, 0, 255), (152, 0, 255), (144, 0, 255),
 (136, 0, 255), (128, 0, 255), (120, 0, 255), (112, 0, 255), (104, 0, 255), (96, 0, 255), (88, 0, 255), (80, 0, 255),
 (72, 0, 255),  (64, 0, 255), (56, 0, 255), (48, 0, 255), (40, 0, 255), (32, 0, 255), (24, 0, 255), (16, 0, 255),
 (8, 0, 255),  (0, 0, 255), (0, 8, 248), (0, 16, 240), (0, 24, 232), (0, 32, 224), (0, 40, 216), (0, 48, 208),
 (0, 56, 200),  (0, 64, 192), (0, 72, 184), (0, 80, 176), (0, 88, 168), (0, 96, 160), (0, 104, 152), (0, 112, 144),
 (0, 120, 136),  (0, 128, 128), (0, 136, 120), (0, 144, 112), (0, 152, 104), (0, 160, 96), (0, 168, 88), (0, 176, 80),
 (0, 184, 72), (0, 192, 64), (0, 200, 56), (0, 208, 48), (0, 216, 40), (0, 224, 32), (0, 232, 24), (0, 240, 16), (0, 248, 8),
 (0, 255, 0), (8, 255, 0), (16, 255, 0), (24, 255, 0), (32, 255, 0), (40, 255, 0), (48, 255, 0), (56, 255, 0), (64, 255, 0),
 (72, 255, 0), (80, 255, 0), (88, 255, 0), (96, 255, 0), (104, 255, 0), (112, 255, 0), (120, 255, 0), (128, 255, 0),
 (136, 255, 0), (144, 255, 0), (152, 255, 0), (160, 255, 0), (168, 255, 0), (176, 255, 0), (184, 255, 0), (192, 255, 0),
 (200, 255, 0), (208, 255, 0), (216, 255, 0), (224, 255, 0), (232, 255, 0), (240, 255, 0), (248, 255, 0), (255, 255, 0),
 (255, 248, 0), (255, 240, 0), (255, 232, 0), (255, 224, 0), (255, 216, 0), (255, 208, 0), (255, 200, 0), (255, 192, 0),
 (255, 184, 0),  (255, 176, 0), (255, 168, 0), (255, 160, 0), (255, 152, 0), (255, 144, 0), (255, 136, 0), (255, 128, 0),
 (255, 120, 0), (255, 112, 0),  (255, 104, 0), (255, 96, 0), (255, 88, 0), (255, 80, 0), (255, 72, 0), (255, 64, 0),
 (255, 56, 0), (255, 48, 0), (255, 40, 0), (255, 32, 0), (255, 24, 0), (255, 16, 0), (255, 8, 0), (255, 0, 0))

#Todo : To be written
#               --StimulusTable

def mooseWriteKkit( modelpath, filename, sceneitems={}):
    global foundmatplotlib_
    if not foundmatplotlib_:
        print('No maplotlib found.'
            '\nThis module can be installed by following command in terminal:'
            '\n\t sudo apt install python-maplotlib', "")
        return False
    else:
        error = ""

        if filename.rfind('.') != -1:
            filename = filename[:filename.rfind('.')]
        else:
            filename = filename[:len(filename)]
        filename = filename+'.g'
        global NA
        NA = 6.0221415e23
        global cmin,cmax,xmin,xmax,ymin,ymax
        cmin, xmin, ymin = 0, 0, 0
        cmax, xmax, ymax = 1, 1, 1
        moose.fixXreacs.restoreXreacs(modelpath)
        compt = moose.wildcardFind(modelpath+'/##[0][ISA=ChemCompt]')
        maxVol = estimateDefaultVol(compt)
        positionInfoExist = True
        if compt:
            if bool(sceneitems):
                cmin,cmax,xmin1,xmax1,ymin1,ymax1 = findMinMax(sceneitems)
            elif not bool(sceneitems):
                srcdesConnection = {}
                setupItem(modelpath,srcdesConnection)
                meshEntry,xmin,xmax,ymin,ymax,positionInfoExist,sceneitems = setupMeshObj(modelpath)
                if not positionInfoExist:
                    #cmin,cmax,sceneitems = autoCoordinates(meshEntry,srcdesConnection)
                    sceneitems = autoCoordinates(meshEntry,srcdesConnection)

            if not positionInfoExist:
                # if position are not from kkit, then zoom factor is applied while
                # writing to genesis. Like if position is from pyqtSceneItem or auto-coordinates
                cmin,cmax,xmin1,xmax1,ymin1,ymax1 = findMinMax(sceneitems)
                for k,v in list(sceneitems.items()):
                    anno = moose.element(k.path+'/info')
                    #x1 = calPrime(v['x'])
                    #y1 = calPrime(v['y'])
                    #sceneitems[k]['x'] = x1
                    #sceneitems[k]['y'] = y1

            f = open(filename, 'w')
            writeHeader (f,maxVol)

            gtId_vol = writeCompartment(modelpath,compt,f)
            errors = ""
            error = writePool(modelpath,f,gtId_vol,sceneitems)
            errors = errors+error

            reacList,error= writeReac(modelpath,f,sceneitems)
            errors = errors+error
            
            enzList,error = writeEnz(modelpath,f,sceneitems)
            errors = errors+error

            chanList, error = writeConcChan(modelpath,f,sceneitems)
            errors = errors+error
            
            error = writeStimulus(modelpath,f)
            errors = errors+error

            error = writeSumtotal(modelpath,f)
            errors = errors+error

            #writeSumtotal(modelpath,f)
            f.write("simundump xgraph /graphs/conc1 0 0 99 0.001 0.999 0\n"
                    "simundump xgraph /graphs/conc2 0 0 100 0 1 0\n")
            tgraphs = moose.wildcardFind(modelpath+'/##[0][ISA=Table2]')
            first, second = " ", " "
            if tgraphs:
                first,second = writeplot(tgraphs,f)
            if first:
                f.write(first)
            f.write("simundump xgraph /moregraphs/conc3 0 0 100 0 1 0\n"
                    "simundump xgraph /moregraphs/conc4 0 0 100 0 1 0\n")
            if second:
                f.write(second)
            f.write("simundump xcoredraw /edit/draw 0 -6 4 -2 6\n"
                    "simundump xtree /edit/draw/tree 0 \\\n"
                    "  /kinetics/#[],/kinetics/#[]/#[],/kinetics/#[]/#[]/#[][TYPE!=proto],/kinetics/#[]/#[]/#[][TYPE!=linkinfo]/##[] \"edit_elm.D <v>; drag_from_edit.w <d> <S> <x> <y> <z>\" auto 0.6\n"
                    "simundump xtext /file/notes 0 1\n")
            storeReacMsg(reacList,f)
        
            storeEnzMsg(enzList,f)
            storeChanMsg(chanList,f)
            if tgraphs:
                storePlotMsgs(tgraphs,f)
            writeFooter1(f)
            writeNotes(modelpath,f)
            writeFooter2(f)
            print('Written to file '+filename)
            return errors, True
        else:
            print("Warning: writeKkit:: No model found on " , modelpath)
            return False

def findMinMax(sceneitems):

    cmin = 0.0
    cmax = 1.0
    xmin,xymin = 0.0,0.0
    xmax,xymax = 1.0,1.0
    xycord = []
    xcord = []
    ycord = []
    for k,v in list(sceneitems.items()):
        xycord.append(v['x'])
        xycord.append(v['y'])
        xcord.append(v['x'])
        ycord.append(v['y'])
    xmin = min(xcord)
    xmax = max(xcord)
    ymin = min(ycord)
    ymax = max(ycord)
    cmin = min(xycord)
    cmax = max(xycord)
    return cmin,cmax,xmin,xmax,ymin,ymax

def calPrime(x):
    prime = int((20*(float(x-cmin)/float(cmax-cmin)))-10)
    return prime

def storeCplxEnzMsgs( enz, f ):
    for sub in enz.neighbors["subOut"]:
        s = "addmsg /kinetics/" + trimPath( moose.element(sub) ) + " /kinetics/" + trimPath(enz) + " SUBSTRATE n \n";
        s = s+ "addmsg /kinetics/" + trimPath( enz ) + " /kinetics/" + trimPath( sub ) +        " REAC sA B \n";
        f.write(s)
    for prd in enz.neighbors["prd"]:
        s = "addmsg /kinetics/" + trimPath( enz ) + " /kinetics/" + trimPath(prd) + " MM_PRD pA\n";
        f.write( s )
    for enzOut in enz.neighbors["enzOut"]:
        s = "addmsg /kinetics/" + trimPath( enzOut ) + " /kinetics/" + trimPath(enz) + " ENZYME n\n";
        s = s+ "addmsg /kinetics/" + trimPath( enz ) + " /kinetics/" + trimPath(enzOut) + " REAC eA B\n";
        f.write( s )

def storeMMenzMsgs( enz, f):
    subList = enz.neighbors["subOut"]
    prdList = enz.neighbors["prd"]
    enzDestList = enz.neighbors["enzDest"]
    for esub in subList:
        es = "addmsg /kinetics/" + trimPath(moose.element(esub)) + " /kinetics/" + trimPath(enz) + " SUBSTRATE n \n";
        es = es+"addmsg /kinetics/" + trimPath(enz) + " /kinetics/" + trimPath(moose.element(esub)) + " REAC sA B \n";
        f.write(es)

    for eprd in prdList:
        es = "addmsg /kinetics/" + trimPath( enz ) + " /kinetics/" + trimPath( moose.element(eprd)) + " MM_PRD pA \n";
        f.write(es)
    for eenzDest in enzDestList:
        enzDest = "addmsg /kinetics/" + trimPath( moose.element(eenzDest)) + " /kinetics/" + trimPath( enz ) + " ENZYME n \n";
        f.write(enzDest)

def storeEnzMsg( enzList, f):
    for enz in enzList:
        enzClass = enz.className
        if (enzClass == "ZombieMMenz" or enzClass == "MMenz"):
            storeMMenzMsgs(enz, f)
        else:
            storeCplxEnzMsgs( enz, f )

def storeChanMsg(chanList,f):
    for channel in chanList:
        for chanOL in channel.neighbors['out']:
            eo = "addmsg /kinetics/" + trimPath(moose.element(channel)) + " /kinetics/" + trimPath(moose.element(chanOL))+ " REAC B A \n";
            eo = eo +"addmsg /kinetics/" + trimPath(moose.element(chanOL)) + " /kinetics/"+trimPath(moose.element(channel))+" PRODUCT n vol \n";
            f.write(eo)
            
        for chanIL in channel.neighbors['in']:
            ei = "addmsg /kinetics/" + trimPath(moose.element(channel)) + " /kinetics/" + trimPath(moose.element(chanIL))+ " REAC A B \n";
            ei = ei +"addmsg /kinetics/" + trimPath(moose.element(chanIL)) + " /kinetics/"+trimPath(moose.element(channel))+" SUBSTRATE n vol \n";
            f.write(ei)
            
        for chanSNC in channel.neighbors['setNumChan']:
            cff =  "addmsg /kinetics/"+trimPath(moose.element(chanSNC))+ " /kinetics/"+trimPath(moose.element(channel))+ " NUMCHAN n \n"
            f.write(cff)

def writeConcChan(modelpath,f,sceneitems):
    error = ""
    concChanList = moose.wildcardFind(modelpath+'/##[0][ISA=ConcChan]')
    for cChan in concChanList:
        if findCompartment(cChan) == moose.element('/'):
            error = error + " \n "+cChan.path+ " doesn't have compartment ignored to write to genesis"
        else:
            x = random.randrange(0,10)
            y = random.randrange(0,10)
            textcolor = ""
            color = ""
            if len(moose.element(cChan).neighbors['setNumChan']) == 1:
                chanParent = moose.element(moose.element(cChan).neighbors['setNumChan'][0])
    
            if not chanParent.isA['PoolBase']:
                print(" raise exception Channel doesn't have pool as parent %s",moose.element(cChan).path)
                return False,"raise exception Channel doesn't have pool as parent"
            else:
                vol = chanParent.volume * NA * 1e-3;
                
                cinfo = cChan.path+'/info'
                if moose.exists(cinfo):
                    x = moose.Annotator(cinfo).getField('x')
                    y = moose.Annotator(cinfo).getField('y')
                    #x = sceneitems[cChan]['x']
                    #y = sceneitems[cChan]['y']
                    color = moose.Annotator(cinfo).getField('color')
                    color = getColorCheck(color,GENESIS_COLOR_SEQUENCE)

                    textcolor = moose.Annotator(cinfo).getField('textColor')
                    textcolor = getColorCheck(textcolor,GENESIS_COLOR_SEQUENCE)
                else:
                    error = error + "\n x and y co-ordinates are not specified for `" + cChan.name+ "` zero will be assigned \n "
                if color == "" or color == " ":
                    color = getRandomColor()
                if textcolor == ""  or textcolor == " ":
                    textcolor = getRandomColor()
                f.write("simundump kchan /kinetics/" + trimPath(cChan)+ " " + str(int(1)) + " " + str(cChan.permeability/1000.0 )+  " " +
                    str(int(0)) + " " +
                    str(int(0)) + " " +
                    str(int(0)) + " " +
                    str(int(0)) + " " +
                    str("") + " " +
                    str(textcolor) + " " + str(color) + " \"\"" +
                    " " + str(int(x)) + " " + str(int(y)) + " "+str(int(0))+"\n")
    
    return concChanList,error
def writeEnz( modelpath,f,sceneitems):
    error = ""
    enzList = moose.wildcardFind(modelpath+'/##[0][ISA=EnzBase]')
    for enz in enzList:
        if findCompartment(enz) == moose.element('/'):
            error = error + " \n "+enz.path+ " doesn't have compartment ignored to write to genesis"
        else:
            x = random.randrange(0,10)
            y = random.randrange(0,10)
            textcolor = ""
            color = ""
            k1 = 0;
            k2 = 0;
            k3 = 0;
            nInit = 0;
            concInit = 0;
            n = 0;
            conc = 0;
            if len(moose.element(enz).neighbors['enzDest']) == 1:
                enzParent = moose.element(moose.element(enz).neighbors['enzDest'][0])
    
            if not enzParent.isA['PoolBase']:
                print(" raise exception enz doesn't have pool as parent %s",moose.element(enz).path)
                return False
            else:
                vol = enzParent.volume * NA * 1e-3;
                isMichaelisMenten = 0;
                enzClass = enz.className
                if (enzClass == "ZombieMMenz" or enzClass == "MMenz"):
                    k1 = enz.numKm
                    k3 = enz.kcat
                    k2 = 4.0*k3;
                    k1 = (k2 + k3) / k1;
                    isMichaelisMenten = 1;

                elif (enzClass == "ZombieEnz" or enzClass == "Enz"):
                    k1 = enz.k1
                    k2 = enz.k2
                    k3 = enz.k3
                    if enz.neighbors['cplx']:
                        cplx = enz.neighbors['cplx'][0]
                        nInit = cplx.nInit[0];
                    else:
                        cplx = moose.Pool(enz.path+"/cplx")
                        moose.Annotator(cplx.path+'/info')
                        moose.connect( enz, 'cplx', cplx, 'reac' )
                        nInit = cplx.nInit

                einfo = enz.path+'/info'
                if moose.exists(einfo):
                    x = sceneitems[enz]['x']
                    y = sceneitems[enz]['y']
                    color = moose.Annotator(einfo).getField('color')
                    color = getColorCheck(color,GENESIS_COLOR_SEQUENCE)

                    textcolor = moose.Annotator(einfo).getField('textColor')
                    textcolor = getColorCheck(textcolor,GENESIS_COLOR_SEQUENCE)
                else:
                    error = error + "\n x and y co-ordinates are not specified for `" + enz.name+ "` zero will be assigned \n "
                if color == "" or color == " ":
                    color = getRandomColor()
                if textcolor == ""  or textcolor == " ":
                    textcolor = getRandomColor()
            
                f.write("simundump kenz /kinetics/" + trimPath(enz) + " " + str(int(0))+  " " +
                    str(concInit) + " " +
                    str(conc) + " " +
                    str(nInit) + " " +
                    str(n) + " " +
                    str(vol) + " " +
                    str(k1) + " " +
                    str(k2) + " " +
                    str(k3) + " " +
                    str(0) + " " +
                    str(isMichaelisMenten) + " " +
                    "\"\"" + " " +
                    str(textcolor) + " " + str(color) + " \"\"" +
                    " " + str(int(x)) + " " + str(int(y)) + " "+str(int(0))+"\n")
    return enzList,error

def nearestColorIndex(color, color_sequence):
    #Trying to find the index to closest color map from the rainbow pickle file for matching the Genesis color map
    distance = [ (color[0] - temp[0]) ** 2 + (color[1] - temp[1]) ** 2 + (color[2] - temp[2]) ** 2
                 for temp in color_sequence]
    minindex = 0
    for i in range(1, len(distance)):
        if distance[minindex] > distance[i] : minindex = i

    return minindex

def storeReacMsg(reacList,f):
    for reac in reacList:
        reacPath = trimPath( reac);
        sublist = reac.neighbors["subOut"]
        prdlist = reac.neighbors["prd"]
        for sub in sublist:
            s = "addmsg /kinetics/" + trimPath( sub ) + " /kinetics/" + reacPath +  " SUBSTRATE n \n";
            s =  s + "addmsg /kinetics/" + reacPath + " /kinetics/" + trimPath( sub ) +  " REAC A B \n";
            f.write(s)

        for prd in prdlist:
            s = "addmsg /kinetics/" + trimPath( prd ) + " /kinetics/" + reacPath + " PRODUCT n \n";
            s = s + "addmsg /kinetics/" + reacPath + " /kinetics/" + trimPath( prd ) +  " REAC B A\n";
            f.write( s)

def writeReac(modelpath,f,sceneitems):
    error = ""
    reacList = moose.wildcardFind(modelpath+'/##[0][ISA=ReacBase]')
    for reac in reacList:
        if findCompartment(reac) == moose.element('/'):
            error = error + " \n "+reac.path+ " doesn't have compartment ignored to write to genesis"
        else:
            color = ""
            textcolor = ""
            kf = reac.numKf
            kb = reac.numKb
            # if sceneitems != None:
            #     value = sceneitems[reac]
            #     x = calPrime(value['x'])
            #     y = calPrime(value['y'])

            
            rinfo = reac.path+'/info'
            if moose.exists(rinfo):
                x = sceneitems[reac]['x']
                y = sceneitems[reac]['y']

                color = moose.Annotator(rinfo).getField('color')
                color = getColorCheck(color,GENESIS_COLOR_SEQUENCE)

                textcolor = moose.Annotator(rinfo).getField('textColor')
                textcolor = getColorCheck(textcolor,GENESIS_COLOR_SEQUENCE)
            else:
                x = 0
                y = 0
                error = error + "\n x and y co-ordinates are not specified for `" + reac.name+ "` zero will be assigned \n "
            if color == "" or color == " ":
                color = getRandomColor()
            if textcolor == ""  or textcolor == " ":
                textcolor = getRandomColor()
            f.write("simundump kreac /kinetics/" + trimPath(reac) + " " +str(0) +" "+ str(kf) + " " + str(kb) + " \"\" " +
                    str(color) + " " + str(textcolor) + " " + str(int(x)) + " " + str(int(y))  + " "+ str(0)+"\n")
    return reacList,error
 
def trimPath(mobj):
    mobj = moose.element(mobj)
    original = mobj
    while not mobj.isA['ChemCompt'] and mobj.path != "/":
        mobj = moose.element(mobj.parent)

    if mobj.path == "/":
        print("%s object doesn't have a parent compartment as a parent." % original)
        return

    # other than the kinetics compartment, all the othername are converted to group in Genesis which are place under /kinetics
    # Any moose object comes under /kinetics then one level down the path is taken.
    # e.g /group/poolObject or /Reac
    if mobj.name != "kinetics":# and ( (mobj.className != "CubeMesh") and (mobj.className != "CylMesh") and (mobj.className != "EndoMesh") and (mobj.className != "NeuroMesh")):
        splitpath = original.path[(original.path.find(mobj.name)):len(original.path)]
    else:
        pos = original.path.find(mobj.name)
        slash = original.path.find('/',pos+1)
        splitpath = original.path[slash+1:len(original.path)]
    splitpath = re.sub(r"\[[0-9]+\]", "", splitpath)
    s = splitpath.replace("_dash_",'-')
    s = splitpath.replace("_space_","_")
    return s

# def writeSumtotal( modelpath,f):
#     funclist = moose.wildcardFind(modelpath+'/##[ISA=Function]')
#     for func in funclist:
#         funcInputs = moose.element(func.path+'/x[0]')
#         s = ""
#         for funcInput in funcInputs.neighbors["input"]:
#             s = s+ "addmsg /kinetics/" + trimPath(funcInput)+ " /kinetics/" + trimPath(moose.element(func.parent)) + " SUMTOTAL n nInit\n"
#         f.write(s)

def writeSumtotal( modelpath,f):
    error = ""
    funclist = moose.wildcardFind(modelpath+'/##[0][ISA=Function]')
    s = ""
    for func in funclist:
        fInfound  = True
        fOutfound = True
        funcInputs = moose.element(func.path+'/x[0]')
        if not len(funcInputs.neighbors["input"]):
            fInfound = False
            error = error +' \n /'+ (moose.element(func)).parent.name+ '/'+moose.element(func).name + ' function doesn\'t have input which is not allowed in genesis. \n This function is not written down into genesis file\n'

        if not len(func.neighbors["valueOut"]):
            error = error +'Function'+func.path+' has not been connected to any output, this function is not written to genesis file'
            fOutfound = False
        else:
            for srcfunc in func.neighbors["valueOut"]:
                if srcfunc.className in ["ZombiePool","ZombieBufPool","Pool","BufPool"]:
                    functionOut = moose.element(srcfunc)
                else:
                    error = error +' \n Function output connected to '+srcfunc.name+ ' which is a '+ srcfunc.className+' which is not allowed in genesis, this function '+(moose.element(func)).path+' is not written to file'
                    fOutfound = False

        if fInfound and fOutfound:
            srcPool = []
            for funcInput in funcInputs.neighbors["input"]:
                if funcInput not in srcPool:
                    srcPool.append(funcInput)
                    if trimPath(funcInput) != None and trimPath(functionOut) != None:
                        s = "addmsg /kinetics/" + trimPath(funcInput)+ " /kinetics/"+ trimPath(functionOut)+ " SUMTOTAL n nInit\n"
                        f.write(s)
                else:
                    error = error + '\n Genesis doesn\'t allow same moluecule connect to function mutiple times. \n Pool \''+ moose.element(funcInput).name + '\' connected to '+ (moose.element(func)).path
    return error

def writeStimulus(modelpath,f):
    error = ""
    if len(moose.wildcardFind(modelpath+'/##[0][ISA=StimulusTable]')):
        error = error +'\n StimulusTable is not written into genesis. This is in Todo List'
    return error

def storePlotMsgs( tgraphs,f):
    s = ""
    if tgraphs:
        for graph in tgraphs:
            slash = graph.path.find('graphs')
            if not slash > -1:
                slash = graph.path.find('graph')
            if slash > -1:
                foundConc = True
                if not ( (graph.path.find('conc1') > -1 ) or
                            (graph.path.find('conc2') > -1 ) or
                            (graph.path.find('conc3') > -1 ) or
                            (graph.path.find('conc4') > -1) ):
                    foundConc = False

                #conc = graph.path.find('conc')
                # if conc > -1 :
                #     tabPath = graph.path[slash:len(graph.path)]
                # else:
                #     slash1 = graph.path.find('/',slash)
                #     tabPath = "/graphs/conc1" +graph.path[slash1:len(graph.path)]
                if foundConc == True:
                    tabPath = "/"+graph.path[slash:len(graph.path)]
                else:
                    slash1 = graph.path.find('/',slash)
                    tabPath = "/graphs/conc1" +graph.path[slash1:len(graph.path)]

                if len(moose.element(graph).msgOut):
                    poolPath = (moose.element(graph).msgOut)[0].e2.path
                    poolEle = moose.element(poolPath)
                    poolName = poolEle.name
                    bgPath = (poolEle.path+'/info')
                    bg = moose.Annotator(bgPath).color
                    bg = getColorCheck(bg,GENESIS_COLOR_SEQUENCE)
                    tabPath = re.sub(r"\[[0-9]+\]", "", tabPath)
                    s = s+"addmsg /kinetics/" + trimPath( poolEle ) + " " + tabPath + \
                            " PLOT Co *" + poolName + " *" + str(bg) +"\n";
    f.write(s)

def writeplot( tgraphs,f ):
    first, second = " ", " "
    if tgraphs:
        for graphs in tgraphs:
            slash = graphs.path.find('graphs')
            if not slash > -1:
                slash = graphs.path.find('graph')
            if slash > -1:
                foundConc = True
                if not ( (graphs.path.find('conc1') > -1 ) or
                            (graphs.path.find('conc2') > -1 ) or
                            (graphs.path.find('conc3') > -1 ) or
                            (graphs.path.find('conc4') > -1) ):
                    foundConc = False
                if foundConc == True:
                    tabPath = "/"+graphs.path[slash:len(graphs.path)]
                else:
                    slash1 = graphs.path.find('/',slash)
                    tabPath = "/graphs/conc1" +graphs.path[slash1:len(graphs.path)]


                if len(moose.element(graphs).msgOut):
                    poolPath = (moose.element(graphs).msgOut)[0].e2.path
                    poolEle = moose.element(poolPath)
                    poolAnno = (poolEle.path+'/info')
                    fg = moose.Annotator(poolAnno).textColor
                    fg = getColorCheck(fg,GENESIS_COLOR_SEQUENCE)
                    tabPath = re.sub(r"\[[0-9]+\]", "", tabPath)
                    if tabPath.find("conc1") >= 0 or tabPath.find("conc2") >= 0:
                        first = first + "simundump xplot " + tabPath + " 3 524288 \\\n" + "\"delete_plot.w <s> <d>; edit_plot.D <w>\" " + str(fg) + " 0 0 1\n"
                    if tabPath.find("conc3") >= 0 or tabPath.find("conc4") >= 0:
                        second = second + "simundump xplot " + tabPath + " 3 524288 \\\n" + "\"delete_plot.w <s> <d>; edit_plot.D <w>\" " + str(fg) + " 0 0 1\n"
    return first,second

def writePool(modelpath,f,volIndex,sceneitems):
    error = ""
    color = ""
    textcolor = ""

    for p in moose.wildcardFind(modelpath+'/##[0][ISA=PoolBase]'):
        if findCompartment(p) == moose.element('/'):
            error += "\n%s doesn't have compartment ignored to write to genesis" % p
        else:
            slave_enable = 0
            if p.isA["BufPool"] or p.isA["ZombieBufPool"]:
                pool_children = p.children
                if pool_children== 0:
                    slave_enable = 4
                else:
                    for pchild in pool_children:
                        if (not pchild.isA["ZombieFunction"]) and (not pchild.isA["Function"]):
                            slave_enable = 4
                        else:
                            slave_enable = 0
                            break

            pp = moose.element(p.parent)
            if (not pp.isA['Enz']) and (not pp.isA['ZombieEnz']):
                # Assuming "p.parent.className !=Enzyme is cplx which is not written to genesis"

                # x = sceneitems[p]['x']
                # y = sceneitems[p]['y']
                # if sceneitems != None:
                #     value = sceneitems[p]
                #     x = calPrime(value['x'])
                #     y = calPrime(value['y'])
                    
                pinfo = p.path+'/info'
                if moose.exists(pinfo):
                    x = sceneitems[p]['x']
                    y = sceneitems[p]['y']
                else:
                    x = 0
                    y = 0
                    error = error  + " \n x and y co-ordinates are not specified for `" + p.name+ "` zero will be assigned"+ " \n "

                if moose.exists(pinfo):
                    color = moose.Annotator(pinfo).getField('color')
                    color = getColorCheck(color,GENESIS_COLOR_SEQUENCE)
                    textcolor = moose.Annotator(pinfo).getField('textColor')
                    textcolor = getColorCheck(textcolor,GENESIS_COLOR_SEQUENCE)
                poolsCmpt = findCompartment(p)
                geometryName = volIndex[float(poolsCmpt.volume)]
                volume = p.volume * NA * 1e-3
                if color == "" or color == " ":
                    color = getRandomColor()
                if textcolor == ""  or textcolor == " ":
                    textcolor = getRandomColor()
                f.write("simundump kpool /kinetics/" + trimPath(p) + " 0 " +
                        str(p.diffConst) + " " +
                        str(0) + " " +
                        str(0) + " " +
                        str(0) + " " +
                        str(p.nInit) + " " +
                        str(0) + " " + str(0) + " " +
                        str(volume)+ " " +
                        str(slave_enable) +
                        " /kinetics"+ geometryName + " " +
                        str(color) +" " + str(textcolor) + " " + str(int(x)) + " " + str(int(y)) + " "+ str(0)+"\n")
    return error

def getColorCheck(color,GENESIS_COLOR_SEQUENCE):
    if isinstance(color, str):
        if color.startswith("#"):
            color = ( int(color[1:3], 16)
                                , int(color[3:5], 16)
                                , int(color[5:7], 16)
                                )
            index = nearestColorIndex(color, GENESIS_COLOR_SEQUENCE)
            index = index/2
            return index
        elif color.startswith("("):
            color = eval(color)[0:3]
            index = nearestColorIndex(color, GENESIS_COLOR_SEQUENCE)
            #This is because in genesis file index of the color is half
            index = index/2
            return index
        else:
            index = color
            return index
    elif isinstance(color, tuple):
        color = [int(x) for x in color[0:3]]
        #color = map(int,color)
        index = nearestColorIndex(color, GENESIS_COLOR_SEQUENCE)
        return index
    elif isinstance(color, int):
        index = color
        return index
    else:
        raise Exception("Invalid Color Value!")

def getRandomColor():
    ignoreColor= ["mistyrose","antiquewhite","aliceblue","azure","bisque","black","blanchedalmond","blue","cornsilk","darkolivegreen","darkslategray","dimgray","floralwhite","gainsboro","ghostwhite","honeydew","ivory","lavender","lavenderblush","lemonchiffon","lightcyan","lightgoldenrodyellow","lightgray","lightyellow","linen","mediumblue","mintcream","navy","oldlace","papayawhip","saddlebrown","seashell","snow","wheat","white","whitesmoke","aquamarine","lightsalmon","moccasin","limegreen","snow","sienna","beige","dimgrey","lightsage"]
    matplotcolor = {}
    for name,hexno in matplotlib.colors.cnames.items():
        matplotcolor[name]=hexno

    k = random.choice(list(matplotcolor.keys()))
    if k in ignoreColor:
        return getRandomColor()
    else:
        return k
def writeCompartment(modelpath,compts,f):
    index = 0
    volIndex = {}
    for compt in compts:
        if compt.name != "kinetics":
            x = xmin+6
            y = ymax+1
            f.write("simundump group /kinetics/" + compt.name + " 0 " + "blue" + " " + "green"       + " x 0 0 \"\" defaultfile \\\n" )
            f.write( "  defaultfile.g 0 0 0 " + str(int(x)) + " " + str(int(y)) + " 0\n")
    i = 0
    l = len(compts)
    geometry = ""
    for compt in compts:
        # if isinstance(compt,moose.CylMesh):
        #     print " 1 "
        #     size = (compt.volume/compt.numDiffCompts)
        # else:    
        size = compt.volume
        ndim = compt.numDimensions
        vecIndex = l-i-1
        i = i+1
        x = xmin+4
        y = ymax+1
        #geometryname = compt.name
        if vecIndex > 0:
            geometry = geometry+"simundump geometry /kinetics" +  "/geometry[" + str(vecIndex) +"] 0 " + str(size) + " " + str(ndim) + " sphere " +" \"\" white black "+ str(int(x)) + " " +str(int(y)) +" 0\n";
            volIndex[float(size)] = "/geometry["+str(vecIndex)+"]"
            #geometry = geometry+"simundump geometry /kinetics/"  +  str(geometryname) + " " +str(size) + " " + str(ndim) + " sphere " +" \"\" white black " + str(int(x)) + " "+str(int(y))+ " 0\n";
            #volIndex[float(size)] = geometryname
        else:
            #geometry = geometry+"simundump geometry /kinetics/"  +  str(geometryname) + " " + str(size) + " " + str(ndim) + " sphere " +" \"\" white black " + str(int(x)) + " "+str(int(y))+ " 0\n";
            geometry = geometry+"simundump geometry /kinetics"  +  "/geometry 0 " + str(size) + " " + str(ndim) + " sphere " +" \"\" white black " + str(int(x)) + " "+str(int(y))+ " 0\n";
            volIndex[float(size)] = "/geometry"
            #volIndex[float(size)] = geometryname
    f.write(geometry)
    writeGroup(modelpath,f)
    return volIndex

def writeGroup(modelpath,f):
    ignore = ["graphs","moregraphs","geometry","groups","conc1","conc2","conc3","conc4","model","data","graph_0","graph_1","graph_2","graph_3","graph_4","graph_5"]
    for g in moose.wildcardFind(modelpath+'/##[0][TYPE=Neutral]'):
        if not g.name in ignore:
            if trimPath(g) != None:
                x = xmin+1
                y = ymax+1
                f.write("simundump group /kinetics/" + trimPath(g) + " 0 " +    "blue" + " " + "green"   + " x 0 0 \"\" defaultfile \\\n")
                f.write("  defaultfile.g 0 0 0 " + str(int(x)) + " " + str(int(y)) + " 0\n")


def writeHeader(f,maxVol):
    simdt = 0.001
    plotdt = 0.1
    rawtime = 100
    maxtime = 100
    defaultVol = maxVol
    f.write("//genesis\n"
            "// kkit Version 11 flat dumpfile\n\n"
            "// Saved on " + str(rawtime)+"\n"
            "include kkit {argv 1}\n"
            "FASTDT = " + str(simdt)+"\n"
            "SIMDT = " +str(simdt)+"\n"
            "CONTROLDT = " +str(plotdt)+"\n"
            "PLOTDT = " +str(plotdt)+"\n"
            "MAXTIME = " +str(maxtime)+"\n"
            "TRANSIENT_TIME = 2"+"\n"
            "VARIABLE_DT_FLAG = 0"+"\n"
            "DEFAULT_VOL = " +str(defaultVol)+"\n"
            "VERSION = 11.0 \n"
            "setfield /file/modpath value ~/scripts/modules\n"
            "kparms\n\n"
            )
    f.write( "//genesis\n"
            "initdump -version 3 -ignoreorphans 1\n"
            "simobjdump table input output alloced step_mode stepsize x y z\n"
            "simobjdump xtree path script namemode sizescale\n"
            "simobjdump xcoredraw xmin xmax ymin ymax\n"
            "simobjdump xtext editable\n"
            "simobjdump xgraph xmin xmax ymin ymax overlay\n"
            "simobjdump xplot pixflags script fg ysquish do_slope wy\n"
            "simobjdump group xtree_fg_req xtree_textfg_req plotfield expanded movealone \\\n"
                    "  link savename file version md5sum mod_save_flag x y z\n"
            "simobjdump geometry size dim shape outside xtree_fg_req xtree_textfg_req x y z\n"
            "simobjdump kpool DiffConst CoInit Co n nInit mwt nMin vol slave_enable \\\n"
                    "  geomname xtree_fg_req xtree_textfg_req x y z\n"
            "simobjdump kreac kf kb notes xtree_fg_req xtree_textfg_req x y z\n"
            "simobjdump kenz CoComplexInit CoComplex nComplexInit nComplex vol k1 k2 k3 \\\n"
                    "  keepconc usecomplex notes xtree_fg_req xtree_textfg_req link x y z\n"
            "simobjdump stim level1 width1 delay1 level2 width2 delay2 baselevel trig_time \\\n"
                    "  trig_mode notes xtree_fg_req xtree_textfg_req is_running x y z\n"
            "simobjdump xtab input output alloced step_mode stepsize notes editfunc \\\n"
                    "  xtree_fg_req xtree_textfg_req baselevel last_x last_y is_running x y z\n"
            "simobjdump kchan perm gmax Vm is_active use_nernst notes xtree_fg_req \\\n"
                    "  xtree_textfg_req x y z\n"
            "simobjdump transport input output alloced step_mode stepsize dt delay clock \\\n"
                    "  kf xtree_fg_req xtree_textfg_req x y z\n"
            "simobjdump proto x y z\n"
            )

def estimateDefaultVol(compts):
    maxVol = 0
    vol = []
    for compt in compts:
        vol.append(compt.volume)
    if len(vol) > 0:
        return max(vol)
    return maxVol

def writeNotes(modelpath,f):
    notes = ""
    #items = moose.wildcardFind(modelpath+"/##[ISA=ChemCompt],/##[ISA=ReacBase],/##[ISA=PoolBase],/##[ISA=EnzBase],/##[ISA=Function],/##[ISA=StimulusTable]")
    items = []
    items = moose.wildcardFind(modelpath+"/##[0][ISA=ChemCompt]") +\
            moose.wildcardFind(modelpath+"/##[0][ISA=PoolBase]") +\
            moose.wildcardFind(modelpath+"/##[0][ISA=ReacBase]") +\
            moose.wildcardFind(modelpath+"/##[0][ISA=EnzBase]") +\
            moose.wildcardFind(modelpath+"/##[0][ISA=Function]") +\
            moose.wildcardFind(modelpath+"/##[0][ISA=StimulusTable]")
    for item in items:
        if not re.search(r"xfer",item.name):
            if not moose.exists(item.path+'/info'):
                continue
            info = item.path+'/info'
            notes = moose.Annotator(info).getField('notes')
            if not notes:
                continue
            m = r'call /kinetics/{0}/notes LOAD \ \n"{1}"\n'.format(
                    trimPath(item), moose.Annotator(info).getField('notes')
                    )
            f.write(m)
            #  f.write("call /kinetics/"+trimPath(item)+"/notes LOAD \ \n\""+moose.Annotator(info).getField('notes')+"\"\n")


def writeFooter1(f):
    f.write("\nenddump\n // End of dump\n")

def writeFooter2(f):
    f.write("complete_loading\n")

if __name__ == "__main__":
    import os
    filename = sys.argv[1]
    filepath, filenameWithext = os.path.split(filename)
    if filenameWithext.find('.') != -1:
        modelpath = filenameWithext[:filenameWithext.find('.')]
    else:
        modelpath = filenameWithext

    moose.loadModel(filename,'/'+modelpath,"gsl")
    output = modelpath+"_.g"
    written = mooseWriteKkit('/'+modelpath,output)

    if written:
            print(" file written to ",output)
    else:
            print(" could be written to kkit format")



# -*- coding: utf-8 -*-
"""test_muparser.py:

Modified from https://elifesciences.org/articles/25827 
Fig4.py
"""

import re
import numpy as np
import moose
import rdesigneur as rd
print("[INFO ] Using moose from %s" % moose.__file__)


def parseExpr(expr, params, hasCa):
    if hasCa:
        expr = expr.replace('Ca', 'x0')
        expr = expr.replace('A', 'x1')
        expr = expr.replace('B', 'x2')
    else:
        expr = expr.replace('Ca', 'x0')  # happens in the negFF model
        expr = expr.replace('A', 'x0')  # This is the usual case.
        expr = expr.replace('B', 'x1')

    parts = re.split('k', expr)
    ret = parts[0]
    for i in parts[1:]:
        ret += str(params['k' + i[:2]])
        ret += i[2:]

    if hasCa:
        return 'x3*( ' + ret + ')'
    else:
        return 'x2*( ' + ret + ')'


def makeChemProto(name, Aexpr, Bexpr, params):
    sw = params['stimWidth']
    diffLength = params['diffusionLength']
    dca = params['diffConstA'] * diffLength * diffLength
    dcb = params['diffConstB'] * diffLength * diffLength

    # Objects
    chem = moose.Neutral('/library/' + name)
    compt = moose.CubeMesh('/library/' + name + '/' + name)
    A = moose.Pool(compt.path + '/A')
    B = moose.Pool(compt.path + '/B')
    Z = moose.BufPool(compt.path + '/Z')
    Ca = moose.BufPool(compt.path + '/Ca')
    phase = moose.BufPool(compt.path + '/phase')
    vel = moose.BufPool(compt.path + '/vel')
    ampl = moose.BufPool(compt.path + '/ampl')
    Adot = moose.Function(A.path + '/Adot')
    Bdot = moose.Function(B.path + '/Bdot')
    CaStim = moose.Function(Ca.path + '/CaStim')
    A.diffConst = dca
    B.diffConst = dcb

    # Equations

    Adot.expr = parseExpr(Aexpr, params, True)
    Bdot.expr = parseExpr(Bexpr, params, False)
    CaStim.expr = 'x2 * exp( -((x0 - t)^2)/(2* ' + str(sw * sw) + ') )'

    # Connections
    Adot.x.num = 4
    moose.connect(Ca, 'nOut', Adot.x[0], 'input')
    moose.connect(A, 'nOut', Adot.x[1], 'input')
    moose.connect(B, 'nOut', Adot.x[2], 'input')
    moose.connect(Z, 'nOut', Adot.x[3], 'input')
    moose.connect(Adot, 'valueOut', A, 'increment')

    Bdot.x.num = 3
    if name[:5] == 'negFF':
        moose.connect(Ca, 'nOut', Bdot.x[0], 'input')
        print('Doing special msg')
    else:
        moose.connect(A, 'nOut', Bdot.x[0], 'input')
    moose.connect(B, 'nOut', Bdot.x[1], 'input')
    moose.connect(Z, 'nOut', Bdot.x[2], 'input')
    moose.connect(Bdot, 'valueOut', B, 'increment')
    CaStim.x.num = 3
    moose.connect(phase, 'nOut', CaStim.x[0], 'input')
    moose.connect(vel, 'nOut', CaStim.x[1], 'input')
    moose.connect(ampl, 'nOut', CaStim.x[2], 'input')
    moose.connect(CaStim, 'valueOut', Ca, 'setN')

    return compt


def makeBis(args=None):
    params = {
        'k0a': 0.1,  # Constant
        'k1a': -5.0,  # Coeff for A
        'k2a': 5.0,  # Coeff for A^2
        'k3a': -1.0,  # Coeff for A^3
        'k4a': 10.0,  # turnon of A by A and Ca
        'k5a': -5.0,  # Turnoff of A by B
        'k1b': 0.01,  # turnon of B by A
        'k2b': -0.01,  # Decay rate of B
        'diffusionLength':
        1.0e-6,  # Diffusion characteristic length, used as voxel length too.
        'dendDiameter': 10e-6,  # Diameter of section of dendrite in model
        'dendLength': 100e-6,  # Length of section of dendrite in model
        'diffConstA': 5.0,  # Diffusion constant of A
        'diffConstB': 2.0,  # Diffusion constant of B
        'stimWidth': 1.0,  # Stimulus width in seconds
        'stimAmplitude': 1.0,  # Stimulus amplitude, arb units. From FHN review
        'blankVoxelsAtEnd': 10,  # of voxels to leave blank at end of cylinder
        'preStimTime': 10.0,  # Time to run before turning on stimulus.
        'postStimTime': 40.0,  # Time to run after stimulus. ~3x decay time
        'settleTime': 20.0,  # Settling time of response, after stimulus.
        # To include in analysis of total response over
        # entire dendrite.
        'fnumber': 1,  # Number to append to fname
    }

    for i in args:
        params[i] = args[i]

    makeChemProto(
        'bis',
        'k0a + k1a*A + k2a*A*A + k3a*A*A*A + k4a*Ca*A/(1+A+10*B) + k5a*A*B',
        'k1b*A*A + k2b*B', params)
    return params


def makeFHN(args=None):
    params = {
        'k_t': 2.5,  # Time-const.
        'k_a': 0.7,  # Coeff1
        'k_b': 0.8,  # Coeff2
        'kxa': 2.0,  # Offset for V for FHN eqns.
        'kxb': 0.8,  # Offset for W for FHN eqns.
        'diffusionLength':
        1.0e-6,  # Diffusion characteristic length, used as voxel length too.
        'dendDiameter': 10e-6,  # Diameter of section of dendrite in model
        'dendLength': 100e-6,  # Length of section of dendrite in model
        'diffConstA': 7.5,  # Diffusion constant of A
        'diffConstB': 5.0,  # Diffusion constant of B
        'stimWidth': 1.0,  # Stimulus width in seconds
        'stimAmplitude': 0.4,  # Stimulus amplitude, arb units. From FHN review
        'blankVoxelsAtEnd': 10,  # of voxels to leave blank at end of cylinder
        'preStimTime': 10.0,  # Time to run before turning on stimulus.
        'postStimTime': 40.0,  # Time to run after stimulus. ~3x decay time
        'settleTime': 20.0,  # Settling time of response, after stimulus.
        # To include in analysis of total response over
        # entire dendrite.
        'fnumber': 1,  # Number to append to fname
    }

    for i in args:
        params[i] = args[i]

    makeChemProto('fhn', '5.0*(A - kxa - (A-kxa)^3/3 - (B-kxb) + Ca)',
                  '(A-kxa + k_a - k_b*(B-kxb))/k_t', params)
    # We do this to get the system to settle at the start.
    B = moose.element('/library/fhn/fhn/B')
    B.nInit = 0.2
    return params


def makeNegFB(args=None):
    params = {
        'k1a': -0.1,  # Coeff for decay of A, slow.
        'k2a': -0.2,  # Coeff for turnoff of A by B, medium.
        'k3a': 10.0,  # turnon of A by Ca, fast.
        'k1b': 0.2,  # turnon of B by A, medium
        'k2b': -0.1,  # Decay rate of B, slow
        'diffusionLength':
        1.0e-6,  # Diffusion characteristic length, used as voxel length too.
        'dendDiameter': 10e-6,  # Diameter of section of dendrite in model
        'dendLength': 100e-6,  # Length of section of dendrite in model
        'diffConstA': 0.5,  # Diffusion constant of A
        'diffConstB': 1.0,  # Diffusion constant of B
        'stimWidth': 1.0,  # Stimulus width in seconds
        'stimAmplitude': 1.0,  # Stimulus amplitude, arb units. From FHN review
        'blankVoxelsAtEnd': 10,  #of voxels to leave blank at end of cylinder
        'preStimTime': 10.0,  # Time to run before turning on stimulus.
        'postStimTime': 40.0,  # Time to run after stimulus. ~3x decay time
        'settleTime': 20.0,  # Settling time of response, after stimulus.
        # To include in analysis of total response over
        # entire dendrite.
        'fnumber': 1,  # Number to append to fname
    }

    for i in args:
        params[i] = args[i]

    makeChemProto('negFB', 'k1a*A + k2a*A*B + k3a*Ca', 'k1b*A + k2b*B', params)
    return params


# Was negFF2 in earlier versions of abstrModelEqns
def makeNegFF(args=None):
    params = {
        'k1a': -0.1,  # Coeff for decay of A, slow.
        'k2a': -0.01,  # Coeff for turnoff of A by B, medium.
        'k3a': 10.0,  # turnon of A by Ca, fast.
        'k4a': 40.0,  # amount of B inhibition of turnon of A by Ca.
        'k1b': 2.0,  # turnon of B by Ca, medium
        'k2b': -0.05,  # Decay rate of B, slow
        'diffusionLength':
        1.0e-6,  # Diffusion characteristic length, used as voxel length too.
        'dendDiameter': 10e-6,  # Diameter of section of dendrite in model
        'dendLength': 100e-6,  # Length of section of dendrite in model
        'diffConstA': 0.02,  # Diffusion constant of A
        'diffConstB': 0.4,  # Diffusion constant of B
        'stimWidth': 1.0,  # Stimulus width in seconds
        'stimAmplitude':
        10.0,  # Stimulus amplitude, arb units. From FHN review
        'blankVoxelsAtEnd': 10,  # of voxels to leave blank at end of cylinder
        'preStimTime': 10.0,  # Time to run before turning on stimulus.
        'postStimTime': 40.0,  # Time to run after stimulus. ~3x decay time
        'settleTime': 20.0,  # Settling time of response, after stimulus.
        # To include in analysis of total response over
        # entire dendrite.
        'fnumber': 1,  # Number to append to fname
    }

    for i in args:
        params[i] = args[i]

    makeChemProto('negFF', 'k1a*A + k2a*A*B + k3a*Ca/(1+k4a*B*B)',
                  'k1b*Ca + k2b*B', params)

    return params


def singleCompt(name, params):
    print('=============')
    print('[INFO] Making compartment %s' % name)
    mod = moose.copy('/library/' + name + '/' + name, '/model')

    A = moose.element(mod.path + '/A')
    Z = moose.element(mod.path + '/Z')
    Z.nInit = 1
    Ca = moose.element(mod.path + '/Ca')
    CaStim = moose.element(Ca.path + '/CaStim')

    print('\n\n[INFO] CaStim %s' % CaStim.path)
    runtime = params['preStimTime'] + params['postStimTime']
    steptime = 50
    CaStim.expr += '+x2*(t>100+' + str(runtime) + ')*(t<100+' + str(
        runtime + steptime) + ')'
    print("[INFO] CaStim.expr = %s" % CaStim.expr)
    tab = moose.Table2('/model/' + name + '/Atab')
    ampl = moose.element(mod.path + '/ampl')
    phase = moose.element(mod.path + '/phase')
    moose.connect(tab, 'requestOut', A, 'getN')
    ampl.nInit = params['stimAmplitude'] * 1
    phase.nInit = params['preStimTime']

    ksolve = moose.Ksolve(mod.path + '/ksolve')
    stoich = moose.Stoich(mod.path + '/stoich')
    stoich.compartment = mod

    stoich.ksolve = ksolve
    stoich.path = mod.path + '/##'
    print('REINIT AND START')
    moose.reinit()
    runtime += 100 + steptime * 2
    moose.start(runtime)
    t = np.arange(0, runtime + 1e-9, tab.dt)
    return name, t, tab.vector


def runPanelDEFG(name, dist, seqDt, numSpine, seq, stimAmpl):
    preStim = 10.0
    blanks = 20
    rdes = rd.rdesigneur(
        useGssa=False,
        turnOffElec=True,
        chemPlotDt=0.1,
        diffusionLength=1e-6,
        cellProto=[['cell', 'soma']],
        chemProto=[['dend', name]],
        chemDistrib=[['dend', 'soma', 'install', '1']],
        plotList=[['soma', '1', 'dend' + '/A', 'n', '# of A']],
    )
    rdes.buildModel()
    A = moose.vec('/model/chem/dend/A')
    Z = moose.vec('/model/chem/dend/Z')
    print(moose.element('/model/chem/dend/A/Adot').expr)
    print(moose.element('/model/chem/dend/B/Bdot').expr)
    print(moose.element('/model/chem/dend/Ca/CaStim').expr)
    phase = moose.vec('/model/chem/dend/phase')
    ampl = moose.vec('/model/chem/dend/ampl')
    vel = moose.vec('/model/chem/dend/vel')
    vel.nInit = 1e-6 * seqDt
    ampl.nInit = stimAmpl
    stride = int(dist) / numSpine
    phase.nInit = 10000
    Z.nInit = 0
    for j in range(numSpine):
        k = int(blanks + j * stride)
        Z[k].nInit = 1
        phase[k].nInit = preStim + seq[j] * seqDt

    moose.reinit()
    runtime = 50
    snapshot = preStim + seqDt * (numSpine - 0.8)
    print(snapshot)
    moose.start(snapshot)
    avec = moose.vec('/model/chem/dend/A').n
    moose.start(runtime - snapshot)
    tvec = []
    for i in range(5):
        tab = moose.element('/model/graphs/plot0[' + str(blanks + i * stride) +
                            ']')
        dt = tab.dt
        tvec.append(tab.vector)
    moose.delete('/model')
    return dt, tvec, avec


def makePassiveSoma(name, length, diameter):
    elecid = moose.Neuron('/library/' + name)
    dend = moose.Compartment(elecid.path + '/soma')
    dend.diameter = diameter
    dend.length = length
    dend.x = length
    return elecid


def run():
    panelC = []
    panelCticks = []
    panelC.append(singleCompt('negFB', makeNegFB([])))
    panelC.append(singleCompt('negFF', makeNegFF([])))
    panelC.append(singleCompt('fhn', makeFHN([])))
    panelC.append(singleCompt('bis', makeBis([])))
    panelCticks.append(np.arange(0, 15.00001, 5))
    panelCticks.append(np.arange(0, 1.50001, 0.5))
    panelCticks.append(np.arange(0, 5.00002, 1))
    panelCticks.append(np.arange(0, 5.00002, 1))
    moose.delete('/model')
    for i in zip(panelC, panelCticks, list(range(len(panelC)))):
        plotPos = i[2] + 5
        doty = i[1][-1] * 0.95
        print('doty', doty)
    print('[INFO] Run is over')


def test_expr_parser():
    moose.Neutral('/library')
    moose.Neutral('/model')
    run()
    print('All done')


if __name__ == '__main__':
    test_expr_parser()



import matplotlib
matplotlib.use('Agg')
import os
import moose
import rdesigneur as rd
import numpy as np

sm_diam = 20e-6
sm_len = 20e-6
dend_diam = 4e-6
dend_len = 500e-6
numDend = 10
sm_area = np.pi*sm_len*sm_diam
elecPlotDt = 50e-6
elecDt = 10e-6
RM = 1
CM = 0.01
RA = 1.5
Vrest = -0.065

sdir_ = os.path.dirname(os.path.realpath(__file__))

def test_proto():
    # Boilerplate
    rdes = rd.rdesigneur(
        elecPlotDt = elecPlotDt,
        elecDt = elecDt,
        cellProto = [['somaProto', 'soma', sm_diam, sm_len]],
        chanProto = [[ os.path.join(sdir_, 'Na_Chan_Migliore2018_.Na_Chan()'), 'Na']],
        chanDistrib = [
            ['Na', 'soma', 'Gbar', '360']
            ],
        passiveDistrib = [
            ['#', 'RM', str(RM), 'CM', str(CM), 'initVm', str(Vrest), 'Em', str(Vrest)],
        ],
        stimList = [['soma', '1', '.', 'vclamp', '-0.065 + (t>0.1 && t<0.2) * 0.02' ]],
        plotList = [
            ['soma', '1', '.', 'Vm', 'Soma membrane potential'],
            ['soma', '1', 'vclamp', 'current', 'Soma holding current'],
        ]
    )
    rdes.buildModel()

    if moose.exists('/model/elec/soma/vclamp'):
        moose.element( '/model/elec/soma/vclamp' ).gain = CM*sm_area/elecDt
        moose.element( '/model/elec/soma/vclamp' ).tau = 5*elecDt
        moose.element( '/model/elec/soma/vclamp' ).ti = elecDt*2
        moose.element( '/model/elec/soma/vclamp' ).td = 0

    moose.reinit()
    moose.start( 0.3 )
    rdes.display()

def main():
    test_proto()

if __name__ == '__main__':
    main()



import numpy as np
import moose
print( '[INFO] Using moose from %s' % moose.__file__ )
import time

# Does not guarantee thread determinism in multithreaded Gsolve/Ksolve.
moose.seed( 10 )

def printCompt(compt):
    print( 'x0=%s, x1=%s, diffLength=%s, numDiffCompt=%d' % (compt.x0, compt.x1,
        compt.diffLength, compt.numDiffCompts))

def test_gsolve_paralllel(nT=4):
    """
    This example implements a reaction-diffusion like system which is
    bistable and propagates losslessly. It is based on the NEURON example 
    rxdrun.py, but incorporates more compartments and runs for a longer time.
    The system is implemented in a function rather than as a proper system
    of chemical reactions. Please see rxdReacDiffusion.py for a variant that 
    uses a reaction plus a function object to control its rates.
    """
    print( 'Using %d threads' % nT )
    dt = 0.1
    # define the geometry
    compt = moose.CylMesh( '/cylinder' )
    compt.r0 = compt.r1 = 100e-9
    compt.x1 = 200e-09
    compt.diffLength = 0.2e-9
    assert( compt.numDiffCompts == compt.x1/compt.diffLength)

    #define the molecule. Its geometry is defined by its parent volume, cylinder
    c = moose.Pool( '/cylinder/pool' )
    c.diffConst = 1e-13 # define diffusion constant

    # Here we set up a function calculation
    func = moose.Function( '/cylinder/pool/func' )
    func.expr = "(-x0*(30e-9-x0)*(100e-9-x0))*0.0001"
    #  func.x.num = 1 #specify number of input variables.

    #Connect the molecules to the func
    moose.connect( c, 'nOut', func.x[0], 'input' )
    #Connect the function to the pool
    moose.connect( func, 'valueOut', c, 'increment' )

    #Set up solvers
    ksolve = moose.Gsolve( '/cylinder/Gsolve' )
    ksolve.numThreads = nT

    dsolve = moose.Dsolve( '/cylinder/dsolve' )
    stoich = moose.Stoich( '/cylinder/stoich' )
    stoich.compartment = compt
    stoich.ksolve = ksolve
    stoich.dsolve = dsolve
    stoich.path = '/cylinder/##'

    #initialize
    x = np.arange( 0, compt.x1, compt.diffLength )
    c.vec.nInit = [ 1000.0 for q in x ]

    # Run and plot it.
    moose.reinit()
    updateDt = 50
    runtime = updateDt * 10
    t1 = time.time()
    res = []
    clk = moose.element( '/clock' )
    for t in range( 0, runtime-1, updateDt ):
        y = c.vec.n
        s = np.sum(y)
        v = (np.mean(y), np.max(y), np.min(y), s)
        print(v)
        res.append(v)
        moose.start( updateDt )
        currTime = clk.currentTime

    # One molecule here and there because thread launching has undeterministic
    # characteristics. Even after setting moose.seed; we may not see same
    # numbers on all platfroms.
    expected = [
            (1000.0, 1000.0, 1000.0, 1000000.0)
            , (9.908, 10.0, 8.0, 9908.0)
            , (6.869, 7.0, 6.0, 6869.0)
            , (5.354, 6.0, 5.0, 5354.0)
            , (4.562, 5.0, 4.0, 4562.0)
            , (3.483, 4.0, 3.0, 3483.0)
            , (3.043, 4.0, 3.0, 3043.0)
            , (2.261, 3.0, 2.0, 2261.0)
            , (1.967, 2.0, 1.0, 1967.0)
            , (1.997, 2.0, 1.0, 1997.0) ]
    print("Time = ", time.time() - t1)
    assert np.isclose(res, expected, atol=1, rtol=1).all(), "Got %s, expected %s" % (res, expected)

def main(nT):
    test_gsolve_paralllel(nT)

if __name__ == '__main__':
    import sys
    import multiprocessing
    nT = max(1, int(multiprocessing.cpu_count()))
    if len(sys.argv) > 1:
        nT = int(sys.argv[1])
    main( nT )




"""streamer_utils.py: 

Utility for Table streamer.

"""
    
__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2017-, Dilawar Singh"
__version__          = "1.0.0"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"
__status__           = "Development"

import sys
import os
import numpy as np
import math
import struct
from collections import defaultdict

def bytes_to_np_arr(data):
    return np.frombuffer(data, float)

def np_array_to_string(arr):
    return np.uint8(arr).tostring()

def np_array_to_data(arr):
    # Make sure that first char of arr is 'H'
    n = 0
    res = defaultdict(list)
    while n < len(arr):
        assert chr(int(arr[n])) == 'H', 'Expected H, Got %s'%arr[n]
        hSize = int(arr[n+1])
        n += 1
        colName = np_array_to_string(arr[n+1:n+1+hSize])
        n += hSize + 1
        assert chr(int(arr[n])) == 'V', 'Expected V'
        n += 1
        dataSize = int(arr[n])
        n += 1
        res[colName].append(arr[n:n+dataSize])
        n += dataSize 
    return { k : np.concatenate(v) for k, v in res.items() }

def decode_data(data):
    arr = bytes_to_np_arr(data)
    assert int(arr[0]) == ord('H'), "First char must be H"
    return np_array_to_data(arr)

def test():
    with open(sys.argv[1], 'rb') as f:
        data = f.read()
    print( "[INFO ] Total bytes read %d" % len(data))
    s = decode_data(data)
    print(s)

if __name__ == '__main__':
    test()



__author__     = "Dilawar Singh"
__copyright__  = "Copyright 2019-, Dilawar Singh"
__maintainer__ = "Dilawar Singh"
__email__      = "dilawars@ncbs.res.in"

import moose

if moose._moose.__generated_by__ != "pybind11":
    print("These bindings are not generated by pybind11.")
    quit(0)

import random
import numpy as np

def test_children():
    a1 = moose.Neutral('/a')
    a2 = moose.Neutral('/a/b')
    a3 = moose.Neutral('/a/b/c1')
    moose.Neutral('/a/b/c2')
    assert len(a1.children) == 1
    assert len(a2.children) == 2
    moose.le(a1)
    moose.le(a2)
    moose.le(a3)
    moose.setCwe(a3)
    s = moose.getCwe()
    assert s == a3, (s, a3)
    a11 = a1.children[0]
    ax = moose.element(a1)
    ax1 = ax.children[0]
    assert ax == a1
    assert ax1 == a11
    assert a11[0].isA['Neutral'], a11.isA
    assert ax1[0].isA['Neutral'], a11.isA
    print("test_children is done")


def test_other():
    a1 = moose.Pool('/ada')
    assert a1.className == 'Pool', a1.className
    finfo = moose.getFieldDict(a1.className)
    s = moose.Streamer('/asdada')
    p = moose.PulseGen('pg1')
    assert p.delay[0] == 0.0
    p.delay[1] = 0.99
    assert p.delay[1] == 0.99, p.delay[1]

    c = moose.Stoich('/dadaa')
    v1 = moose.getFieldNames(c)
    v2 = c.getFieldNames()
    assert v1 == v2
    assert len(v1) > 10, v1
    


def test_vec():
    a = moose.Pool('/p111', 100)
    v = moose.vec(a)

    # le can cause segfault in some cases.
    moose.le(v)

    assert len(v) == 100, len(v)
    assert v == v.vec
    assert v[0] == v.vec[0], (v[0], v.vec[0])
    x = [random.random() for i in range(100)]
    v.conc = x
    assert np.isclose(np.sum(v.conc), sum(x))
    assert np.allclose(v.conc, x), (v.conc, x)

    # assign bool to double.
    y = [float(x < 5) for x in range(100) ]
    v.concInit = y
    assert (v.concInit[:5] == 1.0).all(), v.concInit[:5]
    assert (v.concInit[5:] == 0.0).all(), v.concInit[5:]

def test_finfos():
    s = moose.SimpleSynHandler('synh')

    s.numSynapses = 10
    assert s.numSynapses == 10

    syns = s.synapse.vec
    print(s.synapse, '111')
    s8a = s.synapse[8]
    s8b = s.synapse[-2]
    assert s8a == s8b, (s8a, s8b)

    # negative indexing.
    assert syns[-2] == syns[len(syns)-2]
    assert len(syns) == 10
    for i, s in enumerate(syns):
        s.weight = 9.0
    for s in syns:
        assert s.weight == 9.0

    # this is a shorthand for above for loop.
    syns.weight = 11.121
    assert np.allclose(syns.weight, 11.121), syns.weight

     # try:
     #     print(syns[11])
     # except Exception as e:
     #     print(e, "Great. We must got an exception here")
     # else:
     #     print(syns[11])
     #     raise Exception("This should have failed")

    a = moose.Pool('x13213')
    a.concInit = 0.1
    assert 0.1 == moose.getField(a, 'concInit')

    # Now get some finfos.
    a = moose.element('/classes/Compartment')


def test_inheritance():
    ta = moose.Table2('/tab2', 10)
    tb = moose.wildcardFind('/##[TYPE=Table2]')
    assert len(tb) == len(ta.vec)
    for i, (t1, t2) in enumerate(zip(tb, ta.vec)):
        assert t1 == t2, (t1, t2)
        assert t1.id == t2.id
        assert t1.dataIndex == t2.dataIndex
        assert t1.path == t2.path

    a = moose.CubeMesh('/dadada')
    isinstance(a, moose.CubeMesh)
    assert isinstance(a, moose.CubeMesh)
    aa = moose.wildcardFind('/##[TYPE=CubeMesh]')[0]
    assert a == aa
    # This must be true for isinstance to work.
    assert isinstance(aa, moose.CubeMesh), (a.__class__, aa.__class__)

    a = moose.CubeMesh('yapf')
    assert a.isA('CubeMesh') == a.isA['CubeMesh']
    assert a.isA['CubeMesh']
    assert a.isA['ChemCompt']


def test_delete():
    a = moose.Neutral('/xxx')
    b = moose.Neutral('/xxx/1')
    c = moose.Neutral('/xxx/1/2')
    d = moose.Neutral('/xxx/2')
    e = moose.Neutral('/xxx/2/2')
    f = moose.Neutral('/xxx/2/2/3')
    x = moose.wildcardFind('/xxx/##')
    assert len(x) == 5
    moose.delete(e)
    x = moose.wildcardFind('/xxx/##')
    assert len(x) == 3
    moose.delete(a)
    x = moose.wildcardFind('/xxx/##')
    assert len(x) == 0


def test_wrapper():
    a = moose.Pool('/dadadada', concInit=9.99, nInit=10)
    assert a.nInit == 10
    f = moose.Function('/fun1', expr='x0+x1+A+B')
    assert f.expr == 'x0+x1+A+B'
    assert f.numVars == 4, f.numVars


def test_access():
    a1 = moose.Pool('ac1')
    try:
        a2 = moose.Compartment('ac1')
    except Exception:
        pass
    else:
        raise RuntimeError("Should have failed.")
    a2 = moose.element(a1)
    a3 = moose.element(a1.path)
    assert a2 == a3, (a2, a3)

def test_element():
    a = moose.Pool('xxxx', 2)
    ae = moose.element(a)
    assert ae.parent == a.parent, (ae.parent, a.parent)

def test_typing():
    a = moose.Pool('x123y', 100)

    a.concInit = True
    assert a.concInit == 1.0, a.concInit
    a.concInit = False
    assert a.concInit == 0.0, a.concInit

    av = moose.vec(a)
    av.concInit = 1.0
    assert np.allclose(av.concInit, 1.0), av.concInit
    av.concInit = 0.012
    assert np.allclose(av.concInit, 0.012), av.concInit

    av.concInit = True
    assert np.allclose(av.concInit, 1.0), av.concInit
    
def test_elements():
    a = moose.HHChannel('hhchannel')
    x = a.gateX
    y = a.gateY
    z = a.gateZ
    xe, ye, ze = (moose.element(a) for a in (x, y, z))
    assert xe.isA['HHGate']
    assert ye.isA['HHGate']
    assert ze.isA['HHGate']

def test_paths():
    x = moose.Neutral('///x')
    assert x.path == '/x', x.path

def test_le():
    # see issue BhallaLab/moose-core#423
    x = moose.le('/')
    assert len(x) > 5, x
    try:
        moose.le('/abrakadabra')
    except ValueError:
        pass
    else:
        raise RuntimeError("This should have raised ValueError")

def main():
    test_paths()
    test_children()
    test_finfos()
    test_other()
    test_delete()
    test_wrapper()
    test_inheritance()
    test_access()
    test_element()
    test_vec()
    test_typing()
    test_elements()
    test_le()

if __name__ == '__main__':
    main()



########################################################################
# This example illustrates how a function can be used to control a reaction
# rate. This kind of calculation is appropriate when we need to link 
# different kinds of physical processses with chemical reactions, for  
# example, membrane curvature with molecule accumulation. The use of
# functions to modify reaction rates should be avoided in purely chemical 
# systems since they obscure the underlying chemistry, and do not map
# cleanly to stochastic calculations.
#
# In this example we simply have a molecule C that controls the forward
# rate of a reaction that converts A to B. C is a function of location
# on the cylinder, and is fixed. In more elaborate computations we could
# have a function of multiple molecules, some of which could be changing and
# others could be buffered.
#  
#  Copyright (C) Upinder S. Bhalla NCBS 2018
#  Released under the terms of the GNU Public License V3.
########################################################################

import numpy as np
import moose
import rdesigneur as rd

plot_ = False

def makeFuncRate():
    model = moose.Neutral( '/library' )
    model = moose.Neutral( '/library/chem' )
    compt = moose.CubeMesh( '/library/chem/compt' )
    compt.volume = 1e-15
    A = moose.Pool( '/library/chem/compt/A' )
    B = moose.Pool( '/library/chem/compt/B' )
    C = moose.Pool( '/library/chem/compt/C' )
    reac = moose.Reac( '/library/chem/compt/reac' )
    func = moose.Function( '/library/chem/compt/reac/func' )
    func.x.num = 1
    func.expr = "(x0/1e8)^2"
    moose.connect( C, 'nOut', func.x[0], 'input' )
    moose.connect( func, 'valueOut', reac, 'setNumKf' )
    moose.connect( reac, 'sub', A, 'reac' )
    moose.connect( reac, 'prd', B, 'reac' )

    A.concInit = 1
    B.concInit = 0
    C.concInit = 0
    reac.Kb = 1

def test():
    makeFuncRate()

    rdes = rd.rdesigneur(
            turnOffElec = True,
            #This subdivides the 50-micron cylinder into 2 micron voxels
            diffusionLength = 2e-6,
            cellProto = [['somaProto', 'soma', 5e-6, 50e-6]],
            chemProto = [['chem', 'chem']],
            chemDistrib = [['chem', 'soma', 'install', '1' ]],
            plotList = [['soma', '1', 'dend/A', 'conc', 'A conc', 'wave'],
                ['soma', '1', 'dend/C', 'conc', 'C conc', 'wave']],
    )
    rdes.buildModel()
    ts = moose.wildcardFind('/##[TYPE=Table2]')

    C = moose.element( '/model/chem/dend/C' )
    C.vec.concInit = [ 1+np.sin(x/5.0) for x in range( len(C.vec) ) ]
    moose.reinit()
    moose.start(10)
    if plot_:
        rdes.display()
    ts = moose.wildcardFind( '/##[TYPE=Table2]')
    mat = []
    assert len(ts) == 50, len(ts)

    for t in ts:
        print(t)
        if 'plot1' in t.path:
            mat.append(t.vector)
    mat = np.matrix(mat)

    exMean, exStd = 1.1619681711817156, 0.6944155817587526
    assert np.isclose( np.mean(mat), exMean), (np.mean(mat), exMean)
    assert np.isclose( np.std(mat), exStd), mp.std(mat)
    assert( np.isclose(np.mean(mat, axis=0), exMean).all() )
    assert( np.isclose(np.std(mat, axis=0), exStd).all() )
    assert( np.isclose(0.0, np.std(mat, axis=1)).all())

def main():
    test()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, division

__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2015, Dilawar Singh and NCBS Bangalore"
__credits__          = ["NCBS Bangalore"]
__license__          = "GNU GPL"
__version__          = "1.0.0"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"
__status__           = "Development"

import sys
import os
import moose

# the model lives in the same directory as the test script
modeldir = os.path.dirname(__file__)

import _neuroml
from _neuroml.FvsI_CA1 import ca1_main, loadModel
from _neuroml.CA1 import loadGran98NeuroML_L123

def test_ca1():
    p = os.path.join(modeldir, '_neuroml/cells_channels/CA1soma.morph.xml')
    loadModel(p)
    assert 10 == ca1_main(4e-13)
    assert 20 == ca1_main(8e-13)
    assert 29 == ca1_main(14e-13)
    assert 34 == ca1_main(18e-13)

def test_gran():
    p = os.path.join(modeldir, '_neuroml/CA1soma.net.xml')
    assert loadGran98NeuroML_L123(p) in [8,9]

def main():
    test_ca1()
    test_gran()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from __future__ import print_function

"""test_streamer.py:
Test script for Streamer class.
"""

__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2016, Dilawar Singh"
__credits__          = ["NCBS Bangalore"]
__license__          = "GNU GPL"
__version__          = "1.0.0"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"
__status__           = "Development"

import moose
import threading
import numpy as np
import time
import os
import sys
print('[INFO] Using moose form %s' % moose.__file__)

all_done_ = False

# Poll the file to see that we are really writing to it.
def test_sanity( ):
    a = moose.Table( '/t1' )
    b = moose.Table( '/t1/t1' )
    c = moose.Table( '/t1/t1/t1' )
    print(a)
    print(b)
    print(c)

    st = moose.Streamer( '/s' )

    st.outfile = 'a.txt'
    assert st.outfile == 'a.txt'

    st.addTable( a )
    assert( st.numTables == 1 )
    st.addTable( b )
    assert( st.numTables == 2 )
    st.addTable( c )
    assert( st.numTables == 3 )
    st.addTable( c )
    assert( st.numTables == 3 )
    st.addTable( c )
    assert( st.numTables == 3 )

    st.removeTable( c )
    assert( st.numTables == 2 )
    st.removeTable( c )
    assert( st.numTables == 2 )
    st.removeTable( a )
    assert( st.numTables == 1 )
    st.removeTable( b )
    assert( st.numTables == 0 )
    st.removeTable( b )
    assert( st.numTables == 0 )
    print( 'Sanity test passed' )

    st.addTables( [a, b, c ])
    assert st.numTables == 3
    st.removeTables( [a, a, c] )
    assert st.numTables == 1

def buildSystem(outfile):
    if moose.exists('/compt'):
        moose.delete('/compt')
    compt = moose.CubeMesh( '/compt' )
    assert compt
    r = moose.Reac( '/compt/r' )
    a = moose.Pool( '/compt/a' )
    a.concInit = 1
    b = moose.Pool( '/compt/b' )
    b.concInit = 2
    c = moose.Pool( '/compt/c' )
    c.concInit = 0.5
    moose.connect( r, 'sub', a, 'reac' )
    moose.connect( r, 'prd', b, 'reac' )
    moose.connect( r, 'prd', c, 'reac' )
    r.Kf = 0.1
    r.Kb = 0.01

    tabA = moose.Table2( '/compt/a/tab' )
    tabB = moose.Table2( '/compt/tabB' )
    tabC = moose.Table2( '/compt/tabB/tabC' )
    print(tabA, tabB, tabC)

    moose.connect( tabA, 'requestOut', a, 'getConc' )
    moose.connect( tabB, 'requestOut', b, 'getConc' )
    moose.connect( tabC, 'requestOut', c, 'getConc' )

    # Now create a streamer and use it to write to a stream
    st = moose.Streamer( '/compt/streamer' )
    st.outfile = outfile

    print("outfile set to: %s " % st.outfile )
    st.addTable( tabA )
    st.addTables( [ tabB, tabC ] )
    assert st.numTables == 3
    return st

def test_abit_more():
    stCSV = buildSystem('data.csv')
    moose.reinit( )
    moose.start(100)
    csvData = np.loadtxt(stCSV.outfile, skiprows=1)

    stNumpy = buildSystem('data.npy')
    moose.reinit()
    moose.start(100)
    npData = np.load(stNumpy.outfile)

    assert csvData.shape[0] == npData.shape[0], npData.shape
    assert csvData.shape[1] == len(npData.dtype)   # cols

    for i, name in enumerate(npData.dtype.names):
        assert (csvData[:,i] == npData[name]).all()

def main( ):
    test_sanity( )
    test_abit_more( )

if __name__ == '__main__':
    main()



import moose
import numpy as np
import rdesigneur as rd

def test_chem_osc():
    """Test chemical oscillation.
    """
    rdes = rd.rdesigneur(
        turnOffElec = True,
        diffusionLength = 1e-3, # Default diffusion length is 2 microns
        chemProto = [['makeChemOscillator()', 'osc']],
        chemDistrib = [['osc', 'soma', 'install', '1' ]],
        plotList = [['soma', '1', 'dend/a', 'conc', 'a Conc'],
            ['soma', '1', 'dend/b', 'conc', 'b Conc']]
    )
    rdes.buildModel()
    b = moose.element( '/model/chem/dend/b' )
    b.concInit *= 5
    moose.reinit()
    moose.start(200)
    res = [t.vector for t in moose.wildcardFind('/##[TYPE=Table2]')]
    m = np.mean(res, axis=1)
    s = np.std(res, axis=1)
    expected = ([0.13622532, 0.13261175], [0.08518292, 0.06766806])
    assert np.allclose(expected, [m,s]), (m,s)

if __name__ == '__main__':
    test_chem_osc()



# -*- coding: utf-8 -*-
# optimizer_interface.py ---
#
# Filename: optimizer_interface.py
# Description: Provides an interface between Optimizer and MOOSE
# Author: Viktor Toth
# Maintainer:
# Copyright (C) 2014 Viktor Toth, all rights reserved.
# Created: 7 Aug 14:45:30 2014 (+0530)
# Version: 1.0
# Last-Updated:
# URL:
# Keywords:
# Compatibility:
#
#

# Code:

import os
from . import moose

class OptimizerInterface:
    """
    Establish connection between MOOSE and Optimzer, parameter fitting tool.

    Usage: create an OptimizerInterface object at the beginning of the
    script running the MOOSE simulation. Call getParams() to retrieve
    the parameters advised by Optimizer, then run the simulation using
    these parameters. When the simulation has already run, call addTrace()
    passing every trace as a moose.Table or list of floats. When all the
    traces are added, call writeTraces() so when your script finished
    Optimizer is able to read these traces from traceFile.

    On the second 'layer' of the Optimizer GUI select external
    (as type of simulator) and type into the command text box:
    'python /path_to_your_model_script/script.py 3' if the number of
    parameters to fit is 3.
    """
    paramFile = 'params.param' # contains parameters, separated by newline
    traceFile = 'trace.dat'    # contains traces, TAB separated in columns
    params = []
    traces = []
    def __init__(self, paramFile = paramFile, traceFile = traceFile):
        """
        Opens or create paramFile and traceFile. Reads the parameters
        from paramFile if able to.
        """
        self.paramFile = paramFile
        self.traceFile = traceFile

        # parameter file
        if os.path.isfile(self.paramFile):
            with open(self.paramFile) as f:
                self.params = [float(line) for line in f]
        else:
            open(self.paramFile, 'a').close() # create file

        # trace file
        if not os.path.isfile(self.traceFile):
            open(self.traceFile, 'a').close() # create file

    def addTrace(self, trace):
        """
        A trace can be a moose.Table object or a list of float numbers.
        """
        if isinstance(trace, moose.Table):
            self.traces.append(trace.vec)
        else:
            self.traces.append(trace)

    def writeTraces(self):
        """
        Writes the content of traces to traceFile. Every column is a
        trace separated by TABs.
        """
        # TODO implement interpolation if trace is not the same length - not really needed
        # all traces should have the same length (same sampling freq)
        assert len(self.traces) > 0 and len(self.traces[0]) > 0, 'No traces or empty trace found!'
        for i in range(1, len(self.traces)):
            assert len(self.traces[i - 1]) == len(self.traces[i]), 'All traces should have the same length! Use identical sampling frequency!'

        with open(self.traceFile, 'w') as f:
            for i in range(len(self.traces[0])):
                row = [str(trace[i]) for trace in self.traces]
                f.write('\t'.join(row) + '\n')

    def getParams(self):
        """
        Returns the list of parameters read from paramFile.
        """
        return self.params



# -*- coding: utf-8 -*-
# This script is modified version of GraupnerBrunel2012 model by Aditya Gilra.
# Modification is following:
#  - Added global seed.
#  - Removed some messages.
#  - Added assertion.
#  
# NOTE: This script is used for testing random number generators on various
# platform. This should not be used in any tutorial or scientific demo.

import moose
print( 'Using moose from %s' % moose.__file__ )
import numpy as np

moose.seed( 10 )

def test_GB2012_STDP():
    """
    Simulate a pseudo-STDP protocol and plot the STDP kernel
    that emerges from Ca plasticity of Graupner and Brunel 2012.

    Author: Aditya Gilra, NCBS, Bangalore, October, 2014.
    """

    # ###########################################
    # Neuron models
    # ###########################################

    ## Leaky integrate and fire neuron
    Vrest = -65e-3 # V      # resting potential
    Vt_base = -45e-3 # V    # threshold
    Vreset = -55e-3 # V     # in current steps, Vreset is same as pedestal
    R = 1e8 # Ohm
    tau = 10e-3 # s
    refrT = 2e-3 # s

    # ###########################################
    # Initialize neuron group
    # ###########################################

    ## two neurons: index 0 will be presynaptic, 1 will be postsynaptic
    network = moose.LIF( 'network', 2 );
    moose.le( '/network' )
    network.vec.Em = Vrest
    assert np.allclose(network.vec.Em, Vrest), (network.vec.Em, Vrest)
    network.vec.thresh = Vt_base
    network.vec.refractoryPeriod = refrT
    network.vec.Rm = R
    network.vec.vReset = Vreset
    network.vec.Cm = tau/R
    network.vec.inject = 0.
    network.vec.initVm = Vrest

    tauCa = 20e-3     
    tauSyn = 150.0    
    CaPre = 1.0       
    CaPost = 2.0      
    delayD = 13.7e-3  
    thetaD = 1.0      
    thetaP = 1.3      
    gammaD = 200.0    
    gammaP = 321.808  
    J = 5e-3 # V      
    weight = 0.5      
    bistable = True   

    syn = moose.GraupnerBrunel2012CaPlasticitySynHandler( '/network/syn' )
    syn.numSynapses = 1   
    moose.connect( syn, 'activationOut', network.vec[1], 'activation' )

    # synapse from presynaptic neuron
    moose.connect( network.vec[0],'spikeOut', syn.synapse[0], 'addSpike')

    # post-synaptic spikes also needed for STDP
    moose.connect( network.vec[1], 'spikeOut', syn, 'addPostSpike')

    syn.synapse[0].delay = 0.0
    syn.synapse[0].weight = weight
    syn.CaInit = 0.0
    syn.tauCa = tauCa
    syn.tauSyn = tauSyn
    syn.CaPre = CaPre
    syn.CaPost = CaPost
    syn.delayD = delayD
    syn.thetaD = thetaD
    syn.thetaP = thetaP
    syn.gammaD = gammaD
    syn.gammaP = gammaP
    syn.weightScale = J        
    syn.weightMax = 1.0 
    syn.weightMin = 0.

    syn.noisy = True
    syn.noiseSD = 1.3333
    syn.bistable = bistable

    # ###########################################
    # Setting up tables
    # ###########################################

    Vms = moose.Table( '/plotVms', 2 )
    moose.connect( network, 'VmOut', Vms, 'input', 'OneToOne')
    spikes = moose.Table( '/plotSpikes', 2 )
    moose.connect( network, 'spikeOut', spikes, 'input', 'OneToOne')
    CaTable = moose.Table( '/plotCa', 1 )
    moose.connect( CaTable, 'requestOut', syn, 'getCa')
    WtTable = moose.Table( '/plotWeight', 1 )
    moose.connect( WtTable, 'requestOut', syn.synapse[0], 'getWeight')

    dt = 1e-3 
    moose.useClock( 0, '/network/syn', 'process' )
    moose.useClock( 1, '/network', 'process' )
    moose.useClock( 2, '/plotSpikes', 'process' )
    moose.useClock( 3, '/plotVms', 'process' )
    moose.useClock( 3, '/plotCa', 'process' )
    moose.useClock( 3, '/plotWeight', 'process' )
    moose.setClock( 0, dt )
    moose.setClock( 1, dt )
    moose.setClock( 2, dt )
    moose.setClock( 3, dt )
    moose.setClock( 9, dt )
    moose.reinit()

    # function to make the aPlus and aMinus settle to equilibrium values
    settletime = 10e-3 # s
    def reset_settle():
        """ Call this between every pre-post pair
        to reset the neurons and make them settle to rest.
        """
        syn.synapse[0].weight = weight
        syn.Ca = 0.0
        moose.start(settletime)
        # Ca gets a jump at pre-spike+delayD
        # So this event can occur during settletime
        # So set Ca and weight once more after settletime
        syn.synapse[0].weight = weight
        syn.Ca = 0.0

    # function to inject a sharp current pulse to make neuron spike
    # immediately at a given time step
    def make_neuron_spike(nrnidx,I=1e-7,duration=1e-3):
        """ Inject a brief current pulse to
        make a neuron spike
        """
        network.vec[nrnidx].inject = I
        moose.start(duration)
        network.vec[nrnidx].inject = 0.

    dwlist_neg = []
    ddt = 10e-3 # s
    # since CaPlasticitySynHandler is event based
    # multiple pairs are needed for Ca to be registered above threshold
    # Values from Fig 2, last line of legend
    numpairs = 60           # number of spike parts per deltat
    t_between_pairs = 1.0   # time between each spike pair
    t_extent = 100e-3       # s  # STDP kernel extent,
                            # t_extent > t_between_pairs/2 inverts pre-post pairing!
    # dt = tpost - tpre
    # negative dt corresponds to post before pre
    print('-----------------------------------------------')
    for deltat in np.arange(t_extent,0.0,-ddt):
        reset_settle()
        for i in range(numpairs):
            # post neuron spike
            make_neuron_spike(1)
            moose.start(deltat)
            # pre neuron spike after deltat
            make_neuron_spike(0)
            moose.start(t_between_pairs)  # weight changes after pre-spike+delayD
                                          # must run for at least delayD after pre-spike
        dw = ( syn.synapse[0].weight - weight ) / weight
        print(('post before pre, dt = %1.3f s, dw/w = %1.3f'%(-deltat,dw)))
        dwlist_neg.append(dw)

    print('-----------------------------------------------')
    # positive dt corresponds to pre before post
    dwlist_pos = []
    for deltat in np.arange(ddt,t_extent+ddt,ddt):
        reset_settle()
        for i in range(numpairs):
            # pre neuron spike
            make_neuron_spike(0)
            moose.start(deltat)
            # post neuron spike after deltat
            make_neuron_spike(1)
            moose.start(t_between_pairs)
        dw = ( syn.synapse[0].weight - weight ) / weight
        print(('pre before post, dt = %1.3f s, dw/w = %1.3f'%(deltat,dw)))
        dwlist_pos.append(dw)

    Vmseries0 = Vms.vec[0].vector
    numsteps = len(Vmseries0)

    for t in spikes.vec[0].vector:
        Vmseries0[int(t/dt)-1] = 30e-3 # V

    Vmseries1 = Vms.vec[1].vector

    for t in spikes.vec[1].vector:
        Vmseries1[int(t/dt)-1] = 30e-3 # V

    timeseries = np.linspace(0.,200*numsteps*dt,numsteps)

    # STDP curve
    up, sp = np.mean( dwlist_pos ), np.std( dwlist_pos )
    un, sn = np.mean( dwlist_neg ), np.std( dwlist_neg )

    expected = [0.32476025611655324, 0.22658173497286094,
            0.02706212384326734, -0.2176119329016457, -0.17349820098625146,
            -0.049000627347906, 0.10942145078777199, 0.015381955378225953,
            0.004742824127517586, -0.12298343312253879]
    assert np.isclose(dwlist_pos[1:], expected[1:]).all(), "Got %s \nexpected %s" % (dwlist_pos, expected)

    expected = [-0.07871282492831622, 0.11915009122888964,
            -0.028510348966579557, 0.11812233585111875, 0.05098143255634335,
            -0.2303047508248669, 0.18033418630802123, -0.019377885225611347,
            -0.06038610826728241, 0.06575882890278106]
    assert np.isclose(dwlist_neg[1:], expected[1:]).all(), "Got %s\nexpected %s" % (dwlist_neg,
            expected)


    got = (up, sp)
    expNew = (0.014485615086785508, 0.16206703949072981)
    assert np.isclose(got, expNew).all(), 'Expected: %s, Got: %s' % (str(expNew), str(got))

def main():
    test_GB2012_STDP()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
__author__      =   "HarshaRani"
__credits__     =   ["Upi Lab"]
__license__     =   "GPL3"
__version__     =   "1.0.0"
__maintainer__  =   "HarshaRani"
__email__       =   "hrani@ncbs.res.in"
__status__      =   "Development"
__updated__     =   "Sep 03 2018"


'''
mooseAddChemSolver and mooseDeleteChemSolver is  for adding and deleting only
Chemical solver
'''

import moose
from moose.fixXreacs import fixXreacs

def positionCompt( compt ):
    i = 0
    while (i != len(compt)-1):
        compt[i+1].x1 += compt[i].x1
        compt[i+1].x0 += compt[i].x1
        i += 1

def mooseDeleteChemSolver(modelRoot):
    """Delete solvers from Chemical Compartment    """

    compts = moose.wildcardFind(modelRoot + '/##[ISA=ChemCompt]')
    #if all(isinstance(x, (moose.CubeMesh,moose.CylMesh)) for x in compts):
    if all( ( x.isA["CubeMesh"] or x.isA["CylMesh"])  for x in compts):
        for compt in compts:
            if moose.exists(compt.path + '/stoich'):
                st = moose.element(compt.path + '/stoich')
                st_ksolve = st.ksolve
                st_dsolve = st.dsolve

                moose.delete(st)

                if moose.exists((st_ksolve).path):
                    print("KSolver is deleted for modelpath %s " % st_ksolve)
                    moose.delete(st_ksolve)

                if moose.exists((st_dsolve).path) and st_dsolve.path != '/':
                    print("DSolver is deleted for modelpath %s " % st_dsolve)
                    moose.delete(st_dsolve)
    else:
        return ("mooseDeleteChemSolver is only for deleting Chemical Model solver which has to be `CubeMesh` or `CylMesh` found ",list(set([x.className for x in compts]) - set(['CubeMesh',"CylMesh"])))
        
def stdSolvertype(solverName):
    if solverName.lower() in ["gssa","gillespie","stochastic","gsolve"]:
        return "gssa"
    elif solverName.lower() in ["gsl","runge kutta","deterministic","ksolve","rungekutta","rk5","rkf","rk"]:
        return "gsl"
    elif solverName.lower() in ["ee","exponential euler","exponentialeuler","neutral"]:
        return "ee"
    return "ee"

def mooseAddChemSolver(modelRoot, solver):
    """
    Add the solvers only if all are Chemical compartment
    """
    compts = moose.wildcardFind(modelRoot + '/##[ISA=ChemCompt]')
    #if all(isinstance(x, (moose.CubeMesh,moose.CylMesh)) for x in compts):
    if all( ( x.isA["CubeMesh"] or x.isA["CylMesh"])  for x in compts):
        if not compts:
            return ("Atleast one compartment is required ")
        elif ( len(compts) > 3 ):
            return ("Warning: setSolverOnCompt Cannot handle " ,  len(compts) , " chemical compartments\n")

        else:
            comptinfo = moose.Annotator(moose.element(compts[0]).path + '/info')

            previousSolver = stdSolvertype(comptinfo.solver)
            currentSolver = stdSolvertype(solver)

            if previousSolver != currentSolver:
                comptinfo.solver = currentSolver
                if (moose.exists(compts[0].path + '/stoich')):
                    # "A: and stoich exists then delete the stoich add solver"
                    mooseDeleteChemSolver(modelRoot)
                setCompartmentSolver(modelRoot, currentSolver)
                return True
            else:
                if not moose.exists(compts[0].path + '/stoich'):
                    # " stoich exist, doing nothing"
                    setCompartmentSolver(modelRoot, currentSolver)
                    return True
    else:
        
        return ("mooseAddChemSolver is only for adding Chemical Model which has to be `CubeMesh` or `CylMesh` found ",list(set([x.className for x in compts]) - set(['CubeMesh',"CylMesh"])))


def setCompartmentSolver(modelRoot, solver):
    """
     If Solver type is 'gsl' or 'gssa' do add Solver
     if 'ee' nothing

    """
    if solver != 'ee':
        comptlist = dict((c.volume, c) for c in moose.wildcardFind(modelRoot + '/##[ISA=ChemCompt]'))
        vollist = sorted(comptlist.keys())
        compts = [comptlist[key] for key in vollist]

        #compts = [key for key, value in sorted(comptlist.items(), key=lambda (k,v): (v,k))] 
      
        if (len(compts) >1 ):
            positionCompt(compts)
            fixXreacs( modelRoot )
                
        vollist = sorted(comptlist.keys())
        compts = [comptlist[key] for key in vollist]
        #compts = [key for key, value in sorted(comptlist.items(), key=lambda (k,v): (v,k))] 

        for compt in compts:
            if solver != 'ee':
                if solver.lower() in [ 'gsl', 'runge kutta', 'lsoda' ]:
                    ksolve = moose.Ksolve(compt.path + '/ksolve')
                elif solver.lower() in ['gssa', 'gillespie']:
                    ksolve = moose.Gsolve(compt.path + '/gsolve')

                if (len(compts) > 1):
                    dsolve = moose.Dsolve(compt.path+'/dsolve')

                stoich = moose.Stoich(compt.path + '/stoich')
                stoich.ksolve = ksolve
                if (len(compts) > 1):
                    stoich.dsolve = dsolve

                stoich.compartment = compt
                stoich.reacSystemPath = compt.path + "/##"


        dsolveList = moose.wildcardFind(modelRoot+'/##[ISA=Dsolve]')
        i = 0
        while(i < len(dsolveList)-1):
            dsolveList[i+1].buildMeshJunctions(dsolveList[i])
            i += 1
    if not modelRoot[:1].startswith('/'):
        modelRoot ='/'+modelRoot
    print( " Solver is added to model path `%s` with `%s` solver" % (modelRoot,solver) )



# -*- coding: utf-8 -*-
import numpy
import sys
import os
import moose

scriptdir = os.path.dirname(os.path.realpath(__file__))
print('Script dir %s' % scriptdir)

def test_kkit():

    """This example illustrates loading, running, and saving a kinetic model
    defined in kkit format. It uses a default kkit model but you can specify
    another using the command line ``python filename runtime solver``. We use
    the gsl solver here. The model already defines a couple of plots and sets
    the runtime to 20 seconds.  """

    solver = "gsl"  # Pick any of gsl, gssa, ee..
    mfile = os.path.join(scriptdir, '..', 'data', 'kkit_objects_example.g')
    runtime = 20.0
    modelId = moose.loadModel(mfile, 'model')
    moose.reinit()
    moose.start(runtime)

    for x in moose.wildcardFind('/model/#graphs/conc#/#'):
        t = numpy.arange(0, x.vector.size, 1) * x.dt

    vals = x.vector
    stats = [vals.min(), vals.max(), vals.mean(), vals.std()]
    expected = [0.0, 0.00040464, 0.0001444, 0.00013177]
    assert numpy.allclose(stats, expected,
                          rtol=1e-4), 'Got %s expected %s' % (stats, expected)

def main():
    test_kkit()

# Run the 'main' if this script is executed standalone.
if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
import sys
import numpy as np
import time
import moose

print('Using moose from %s' % moose.__file__ )

def test_ksolve():
    compt = moose.CubeMesh( '/compt' )
    compt.volume = 1e-20

    pools = []
    for r in range( 10 ):
        a1 = moose.Pool( '/compt/a1%s' % r )
        a1.concInit = 10
        a2 = moose.Pool( '/compt/a2%s' % r )
        a2.concInit = 5
        b1 = moose.Pool( '/compt/b1%s' % r )
        b1.concInit = 0.054
        b2 = moose.Pool( '/compt/b2%s' % r )
        b2.concInit = 3.9
        r = moose.Reac( '/compt/reac%s'% r )
        moose.connect( r, 'sub', a1, 'reac' )
        moose.connect( r, 'sub', a2, 'reac' )
        moose.connect( r, 'prd', b1, 'reac' )
        moose.connect( r, 'prd', b2, 'reac' )
        r.Kf = 2.9
        r.Kb = 4.5
        pools += [ a1, a2, b1, b2 ]

    ksolve = moose.Ksolve( '/compt/ksolve' )
    stoich = moose.Stoich( '/compt/stoich' )
    stoich.compartment = compt
    stoich.ksolve = ksolve
    ksolve.numThreads = 2
    stoich.path = '/compt/##'
    moose.reinit()
    print( '[INFO] Using method = %s' % ksolve.method )
    t1 = time.time()
    moose.start( 100 )
    print('[INFO] Time taken %s' % (time.time() - t1 ))
    expected = [ 7.77859 , 2.77858 , 2.27541 , 6.12141 , 7.77858 , 2.77858
            , 2.27541 , 6.12141 , 7.77858 , 2.77858 , 2.27541 , 6.12141 , 7.77858
            , 2.77858 , 2.27541 , 6.12141 , 7.77858 , 2.77858 , 2.27541 , 6.12141
            , 7.77858 , 2.77858 , 2.27541 , 6.12141 , 7.77858 , 2.77858 , 2.27541
            , 6.12141 , 7.77858 , 2.77858 , 2.27541 , 6.12141 , 7.77858 , 2.77858
            , 2.27541 , 6.12141 , 7.77858 , 2.77858 , 2.27541 , 6.12141
            ]
    concs = [ p.conc for p in pools ]
    if(not np.isclose( concs, expected ).all() ):
        print( " Expected %s" % expected )
        print( " Got %s" % concs )
        quit(1)
    print( 'Test passed' )

if __name__ == '__main__':
    test_ksolve()



# -*- coding: utf-8 -*-
"""test_table_stream.py:

    Test moose.TableStream.

    This class takes over moose.Table and make sure that their vector is written
    to stream. Once a chunk of data is written, it is removed from vector.

    This way, moose never runs out of memory during simulation.

"""
__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2016, Dilawar Singh"
__credits__          = ["NCBS Bangalore"]
__license__          = "GNU GPL"
__version__          = "1.0.0"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"
__status__           = "Development"

import sys
import moose

print( '[INFO] Using moose from %s' % moose.__file__ )

t1 = moose.Table( '/t1' )
t2 = moose.Table( '/t1/t1' )

a = moose.Streamer( '/a' )
assert( a.streamname == 'stdout' ), 'default is stdout, got %s' % a.streamname

# Add another table
a.addTable( t1 )
a.addTable( t2 )
a.removeTable( t1 )



# -*- coding: utf-8 -*-
from __future__ import print_function

from datetime import datetime
import moose

def time_creation(n=1000):
    elist = []
    start = datetime.now()
    for ii in range(n):
        elist.append(moose.Neutral('a_%d' % (ii)))
    end = datetime.now()
    delta = end - start
    print('total time to create %d Neutral elements: %g' % (n, delta.days * 86400 + delta.seconds + delta.microseconds * 1e-6))
    return delta

if __name__ == '__main__':
    time_creation()



from __future__ import absolute_import, print_function, division
# rmoogli.py: rdesigneur Moogli interface
# This is a fallback version designed to work with moogul but using almost
# the same interface as far as rdesigneur is concerned.
# Put in because the GL versions like moogli need all sorts of difficult 
# libraries and dependencies.
# Copyright (C) Upinder S. Bhalla NCBS 2018
# This program is licensed under the GNU Public License version 3.

import rdesigneur.moogul as moogul
mooViews = []

def makeMoogli( rd, mooObj, args, fieldInfo ):
    #mooObj is currently poorly handled. Ideally it should simply be a 
    # vector of objects to plot, each with coords. This could be readily
    # used to build up the display in a neutral manner.
    # Another version is to have a vector of objects to plot, only as a 
    # way to get at their fields. We would separately need mapping to
    # neuron compartments and index along length.
    # Cleaner still would be to have the C code give a vector of values
    # For now it means something different for chem and elec displays.
    #moogliEntry = [elecPath,bool,whichObjToDisplay,FieldToDisplay,titleForDisplay,rangeMin,rangeMax]
    mooField = args.field
    relObjPath = args.relpath
    numMoogli = len( mooObj )
    if args.ymin != args.ymax:
        ymin = args.ymin
        ymax = args.ymax
    else:
        ymin = fieldInfo[4]
        ymax = fieldInfo[5]
    #print( "fieldinfo = {}, ymin = {}, ymax = {}".format( fieldInfo, ymin, ymax ))

    viewer = moogul.MooView()
    if mooField == 'n' or mooField == 'conc':
        #moogul.updateDiffCoords( mooObj )
        reacSystem = moogul.MooReacSystem( mooObj, fieldInfo, 
                field = mooField, relativeObj = relObjPath, 
                valMin = ymin, valMax = ymax )
        viewer.addDrawable( reacSystem )
    else:
        neuron = moogul.MooNeuron( rd.elecid, fieldInfo,
                field = mooField, relativeObj = relObjPath,
                valMin = ymin, valMax = ymax )
        print( "min = {}, max = {}".format(ymin, ymax) )
        viewer.addDrawable( neuron )

    return viewer

def updateMoogliViewer():
    for i in mooViews:
        i.updateValues()
    

def displayMoogli( rd, _dt, _runtime, rotation = 0.0, fullscreen = False, azim = 0.0, elev = 0.0 ):
    global mooViews
    mooViews = rd.moogNames
    for i in rd.moogNames:
        i.firstDraw( rotation = rotation, azim = azim, elev = elev ) 
        # rotation in radians/frame, azim, elev in radians.




# -*- coding: utf-8 -*-
from __future__ import print_function, division

# This example demonstrates random (Poisson) synaptic input to a cell.
# Copyright (C) Upinder S. Bhalla NCBS 2018
# Released under the terms of the GNU Public License V3. No warranty.
# Changelog:
# Thursday 20 September 2018 09:53:27 AM IST
# - Turned into a test script. Dilawar Singh <dilawars@ncbs.res.in>

import moose
import numpy as np

moose.seed( 100 )

try:
    import matplotlib 
except Exception as e:
    print( "[INFO ] matplotlib is not found. This test wont run." )
    quit()

import rdesigneur as rd

def test_rdes():
    rdes = rd.rdesigneur(
        cellProto = [['somaProto', 'soma', 20e-6, 200e-6]],
        chanProto = [['make_glu()', 'glu']],
        chanDistrib = [['glu', 'soma', 'Gbar', '1' ]],
        stimList = [['soma', '0.5', 'glu', 'randsyn', '50' ]],
        # Deliver stimulus to glu synapse on soma, at mean 50 Hz Poisson.
        plotList = [['soma', '1', '.', 'Vm', 'Soma membrane potential']]
    )
    rdes.buildModel()
    moose.reinit()
    moose.start( 0.3 )
    tables = moose.wildcardFind( '/##[TYPE=Table]' )
    res = [ ]
    for t in tables:
        y = t.vector
        u, s = np.mean(y), np.std(y)
        res.append( (u,s) )


    # Got these values from version compiled on Sep 20, 2018 with moose.seed
    # set to 100.
    expected = [(-0.051218660048699974, 0.01028490481294165)]
    assert np.isclose( expected, res, atol=1e-5).all(), "Expected %s, got %s" %(expected,res)

def main():
    test_rdes()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#########################################################################
## This program is part of 'MOOSE', the
## Messaging Object Oriented Simulation Environment.
##           Copyright (C) 2014 Upinder S. Bhalla. and NCBS
## It is made available under the terms of the
## GNU Lesser General Public License version 2.1
## See the file COPYING.LIB for the full notice.
## Here we set up equal Kf and Kb in a reversible reaction across a
## 1-voxel cyl compt and an internal endoMesh, having a smaller volume.
##                        \
##    COMPT:    2 s <===> / 2 s  :ENDO
##                        \
##
## This becomes:
##                                      \
##    COMPT:    2 s <===> 2 s_xfer_endo / 2s     :ENDO
##                                      \
##
#########################################################################


import math
import numpy as np
import moose
import moose.fixXreacs as fixXreacs

def makeModel():
    # create container for model
    num = 1 # number of compartments
    model = moose.Neutral( '/model' )
    compartment = moose.CylMesh( '/model/compartment' )
    compartment.x1 = 1.0e-6 # Set it to a 1 micron single-voxel cylinder

    # create molecules and reactions
    s = moose.Pool( '/model/compartment/s' )
    rXfer = moose.Reac( '/model/compartment/rXfer' )
    #####################################################################
    # Put in endo compartment. Add molecule s
    endo = moose.EndoMesh( '/model/endo' )
    endo.isMembraneBound = True
    endo.surround = compartment
    es = moose.Pool( '/model/endo/s' )
    #####################################################################
    moose.connect( rXfer, 'sub', s, 'reac' )
    moose.connect( rXfer, 'sub', s, 'reac' )
    moose.connect( rXfer, 'prd', es, 'reac' )
    moose.connect( rXfer, 'prd', es, 'reac' )
    rXfer.Kf = 0.01 # 0.01/sec
    rXfer.Kb = 0.01 # 0.01/sec

    #####################################################################
    fixXreacs.fixXreacs( '/model' )
    fixXreacs.restoreXreacs( '/model' )
    fixXreacs.fixXreacs( '/model' )
    #####################################################################

    # Make solvers
    ksolve = moose.Ksolve( '/model/compartment/ksolve' )
    dsolve = moose.Dsolve( '/model/dsolve' )
    eksolve = moose.Ksolve( '/model/endo/ksolve' )
    edsolve = moose.Dsolve( '/model/endo/dsolve' )

    stoich = moose.Stoich( '/model/compartment/stoich' )
    stoich.compartment = compartment
    stoich.ksolve = ksolve
    stoich.dsolve = dsolve
    stoich.path = "/model/compartment/##"
    assert( dsolve.numPools == 2 )
    s.vec.concInit = [1.0]*num

    estoich = moose.Stoich( '/model/endo/stoich' )
    estoich.compartment = endo
    estoich.ksolve = eksolve
    estoich.dsolve = edsolve
    estoich.path = "/model/endo/##"
    assert( edsolve.numPools == 1 )
    edsolve.buildMeshJunctions( dsolve )

def almostEq( a, b ):
    return abs(a-b)/(a+b) < 5e-5

def test_xreac8():
    runtime = 100
    makeModel()
    moose.reinit()
    moose.start( runtime )
    assert( almostEq( moose.element( 'model/compartment/s' ).conc,
        moose.element( '/model/endo/s' ).conc ) )

def main():
    test_xreac8()


# Run the 'main' if this script is executed standalone.
if __name__ == '__main__':
    main()
